#!/usr/bin/env python3
#############################################################################
### See: http://www.brain-map.org/api/index.html
### http://help.brain-map.org/display/api/RESTful+Model+Access+%28RMA%29
### 
###  http://api.brain-map.org/api/v2/data/[Model]/[Model.id].[json|xml|csv]
###
###  http://api.brain-map.org/api/v2/data/Organism/1.xml
###  http://api.brain-map.org/api/v2/data/Gene/15.xml
###  http://api.brain-map.org/api/v2/data/Chromosome/12.json
###  http://api.brain-map.org/api/v2/data/Structure/4005.xml
###
### http://api.brain-map.org/api/v2/data/Organism/query.json
### http://api.brain-map.org/api/v2/data/Gene/describe.json
### http://api.brain-map.org/api/v2/data/enumerate.json
###
###  http://api.brain-map.org/api/v2/data/Gene/
###  http://api.brain-map.org/api/v2/data/Gene/18376.json
### 
###  &num_rows=[#]&start_row=[#]&order=[...]
#############################################################################
import sys,os,re,argparse,time,json,logging

from ... import allen
#
##############################################################################
if __name__=='__main__':
  API_HOST='api.brain-map.org'
  API_BASE_PATH='/api/v2'
  PROG=os.path.basename(sys.argv[0])
  ftype='SYMBOL';
  ops = ["show_info", "list_probes"]
  parser = argparse.ArgumentParser( description='AllenBrainAtlas REST API client')
  parser.add_argument("op", choices=ops, help='OPERATION (select one)')
  parser.add_argument("--i", dest="ifile", help="input file")
  parser.add_argument("--ids", help="input IDs")
  parser.add_argument("--o", dest="ofile", help="output (TSV)")
  parser.add_argument("--api_host", default=API_HOST)
  parser.add_argument("--api_base_path", default=API_BASE_PATH)
  parser.add_argument("-v", "--verbose", action="count", default=0)
  args = parser.parse_args()

  logging.basicConfig(format='%(levelname)s:%(message)s', level=(logging.DEBUG if args.verbose>1 else logging.INFO))

  api_base_url='https://'+args.api_host+args.api_base_path

  fout = open(args.ofile, "w") if args.ofile else sys.stdout

  ids=[]
  if args.ifile:
    fin = open(args.ifile)
    while True:
      line = fin.readline()
      if not line: break
      ids.append(line.rstrip())
    fin.close()
  elif args.ids:
    ids = re.split('[, ]+', args.ids.strip())
  if len(ids)>0: logging.info('Input IDs: %d'%(len(ids)))

  t0=time.time()

  if args.op=="show_info":
    allen.brain.Utils.ShowInfo(api_base_url, fout)

  elif args.op=="list_probes":
    allen.brain.Utils.ListProbes(api_base_url, ids, fout)

  else:
    parser.error('Invalid operation: {0}'.format(args.op))

  logging.info('Elapsed time: %s'%(time.strftime('%Hh:%Mm:%Ss', time.gmtime(time.time()-t0))))



#!/usr/bin/env python3
#############################################################################
### CMap Clue API client.
### CMap is the project; Clue is the platform.
### https://clue.io/api
### https://clue.io/connectopedia/query_api_tutorial
### https://clue.io/connectopedia/perturbagen_types_and_controls
### l1000_type: "landmark"|"inferred"|"best inferred"|"not inferred"
### BING = Best inferred plus Landmark
### API services: cells, genes, perts, pcls, plates, profiles, sigs,
### probeset_to_entrez_id, rep_fda_exclusivity
#############################################################################
### Apps:
### https://clue.io/cell-app
### https://clue.io/repurposing-app
#############################################################################
### curl -X GET --header "user_key: YOUR_KEY_HERE" 'https://api.clue.io/api/cells?filter=\{"where":\{"provider_name":"ATCC"\}\}' |python3 -m json.tool
#############################################################################
import sys,os,re,argparse,json,logging
import pandas as pd
import urllib.parse
#
from ...util import rest
#
API_HOST="api.clue.io"
API_BASE_PATH="/api"
BASE_URL='https://'+API_HOST+API_BASE_PATH
N_CHUNK=200
#
#############################################################################
def ListDatatypes(params, base_url=BASE_URL, fout=None):
  tags=None; df=pd.DataFrame();
  headers = {"Accept":"application/json", "user_key":params['user_key']}
  url = base_url+'/dataTypes'
  rval = rest.Utils.GetURL(url, headers=headers, parse_json=True)
  logging.debug(json.dumps(rval, indent=2))
  dts = rval
  for dt in dts:
    if not tags:
      tags = [tag for tag in dt.keys() if type(dt[tag]) not in (list, dict)]
    df = pd.concat([df, pd.DataFrame({tags[j]:[dt[tags[j]]] for j in range(len(tags))})])
  if fout: df.to_csv(fout, "\t", index=False)
  logging.info(f"n_datatype: {df.shape[0]}")
  return df

#############################################################################
def ListDatasets(params, base_url=BASE_URL, fout=None):
  tags=None; df=pd.DataFrame();
  headers = {"Accept":"application/json", "user_key":params['user_key']}
  url = (base_url+'/datasets')
  rval = rest.Utils.GetURL(url, headers=headers, parse_json=True)
  logging.debug(json.dumps(rval, indent=2))
  dsets = rval
  for dset in dsets:
    if not tags:
      tags = [tag for tag in dset.keys() if type(dset[tag]) not in (list, dict)]
    df = pd.concat([df, pd.DataFrame({tags[j]:[dset[tags[j]]] for j in range(len(tags))})])
  if fout: df.to_csv(fout, "\t", index=False)
  logging.info(f"n_dataset: {df.shape[0]}")
  return df

#############################################################################
def ListPerturbagenClasses(params, base_url=BASE_URL, fout=None):
  tags=None; df=pd.DataFrame();
  headers = {"Accept":"application/json", "user_key":params['user_key']}
  url = (base_url+'/pcls')
  pcls = rest.Utils.GetURL(url, headers=headers, parse_json=True)
  for pcl in pcls:
    if not tags: tags = list(pcl.keys())
    df = pd.concat([df, pd.DataFrame({tags[j]:[pcl[tags[j]]] for j in range(len(tags))})])
  if fout: df.to_csv(fout, "\t", index=False)
  logging.info(f"n_pertclasses: {df.shape[0]}")
  return df

#############################################################################
def GetGenes(params, ids, id_type, base_url=BASE_URL, fout=None):
  tags=None; df=pd.DataFrame();
  url_base = (base_url+'/genes?user_key='+params['user_key'])
  for id_this in ids:
    logging.debug('ID: '+id_this)
    i_chunk=0;
    while True:
      qry = ('{"where":{"%s":"%s"},"skip":%d,"limit":%d}'%(id_type, urllib.parse.quote(id_this), i_chunk*N_CHUNK, N_CHUNK))
      url = url_base+('&filter={}'.format(urllib.parse.quote(qry)))
      genes = rest.Utils.GetURL(url, parse_json=True)
      if not genes: break
      for gene in genes:
        if not tags: tags = list(gene.keys())
        df = pd.concat([df, pd.DataFrame({tags[j]:[gene[tags[j]]] for j in range(len(tags))})])
      i_chunk+=1
  if fout: df.to_csv(fout, "\t", index=False)
  logging.info(f"IDs: {len(ids)}")
  logging.info(f"n_gene: {df.shape[0]}")
  return df

#############################################################################
def ListGenes_Landmark(params, base_url=BASE_URL, fout=None):
  return GetGenes(params, ['landmark'], 'l1000_type', base_url, fout)

#############################################################################
def ListGenes(params, base_url=BASE_URL, fout=None):
  return GetGenes(params, ['landmark', 'inferred', 'best inferred', 'not inferred'], 'l1000_type', base_url, fout)

#############################################################################
### pert_type:
###	trt_cp - Compound
###	trt_sh - shRNA for loss of function (LoF) of gene
###	trt_lig - Peptides and other biological agents (e.g. cytokine)
###	trt_sh.cgs - Consensus signature from shRNAs targeting the same gene
#############################################################################
def GetPerturbagens(params, ids, id_type, base_url=BASE_URL, fout=None):
  tags=None; df=pd.DataFrame();
  headers = {"Accept":"application/json", "user_key":params['user_key']}
  url_base = (base_url+'/perts')
  fields = ['pert_id', 'pert_iname', 'pert_type', 'pert_vendor', 'pert_url', 'id', 'pubchem_cid', 'entrez_geneId', 'vector_id', 'clone_name', 'oligo_seq', 'description', 'target', 'structure_url', 'moa', 'pcl_membership', 'tas', 'num_sig', 'status']
  for id_this in ids:
    logging.debug('ID: '+id_this)
    i_chunk=0;
    while True:
      qry = ('{"where":{"%s":"%s"},"fields":[%s],"skip":%d,"limit":%d}'%(id_type, urllib.parse.quote(id_this), (','.join(['"%s"'%f for f in fields])), i_chunk*N_CHUNK, N_CHUNK))
      url = url_base+('?filter={}'.format(urllib.parse.quote(qry)))
      perts = rest.Utils.GetURL(url, headers=headers, parse_json=True)
      if not perts: break
      logging.debug(json.dumps(perts, indent=2))
      for pert in perts:
        if not tags: tags = list(pert.keys())
        df = pd.concat([df, pd.DataFrame({tags[j]:[pert[tags[j]]] for j in range(len(tags))})])
      i_chunk+=1
  if fout: df.to_csv(fout, "\t", index=False)
  logging.info(f"IDs: {len(ids)}")
  logging.info(f"n_pert: {df.shape[0]}")
  return df

#############################################################################
def ListPerturbagens(params, base_url=BASE_URL, fout=None):
  pert_types = ['trt_cp', 'trt_lig', 'trt_sh', 'trt_sh.cgs', 'trt_oe', 'trt_oe.mut', 'trt_xpr', 'trt_sh.css', 'ctl_vehicle.cns', 'ctl_vehicle', 'ctl_vector', 'ctl_vector.cns', 'ctl_untrt.cns', 'ctl_untrt']
  return GetPerturbagens(params, pert_types, 'pert_type', base_url, fout)

#############################################################################
def ListDrugs(params, base_url=BASE_URL, fout=None):
  tags=None; df=pd.DataFrame();
  headers = {"Accept":"application/json", "user_key":params['user_key']}
  url_base = (base_url+'/rep_drugs')
  i_chunk=0;
  while True:
    qry = ('{"skip":%d,"limit":%d}'%(i_chunk*N_CHUNK, N_CHUNK))
    url = url_base+('?filter={}'.format(urllib.parse.quote(qry)))
    drugs = rest.Utils.GetURL(url, headers=headers, parse_json=True)
    if not drugs: break
    for drug in drugs:
      if not tags: tags = list(drug.keys())
      df = pd.concat([df, pd.DataFrame({tags[j]:[drug[tags[j]]] for j in range(len(tags))})])
    i_chunk+=1
  if fout: df.to_csv(fout, "\t", index=False)
  logging.info(f"IDs: {len(ids)}")
  logging.info(f"n_drug: {df.shape[0]}")
  return df

#############################################################################
def GetCells(params, ids, id_type, base_url=BASE_URL, fout=None):
  tags=None; df=pd.DataFrame();
  headers = {"Accept":"application/json", "user_key":params['user_key']}
  url_base = (base_url+'/cells')
  n_cell=0;
  for id_this in ids:
    logging.debug('ID: '+id_this)
    i_chunk=0;
    while True:
      qry = ('{"where":{"%s":"%s"},"skip":%d,"limit":%d}'%(id_type, urllib.parse.quote(id_this), i_chunk*N_CHUNK, N_CHUNK))
      url = url_base+('?filter={}'.format(urllib.parse.quote(qry)))
      cells = rest.Utils.GetURL(url, headers=headers, parse_json=True)
      if not cells: break
      logging.debug(json.dumps(cells, indent=2))
      for cell in cells:
        if not tags: tags = list(cell.keys())
        df = pd.concat([df, pd.DataFrame({tags[j]:[cell[tags[j]]] for j in range(len(tags))})])
      i_chunk+=1
  if fout: df.to_csv(fout, "\t", index=False)
  logging.info(f"IDs: {len(ids)}")
  logging.info(f"n_cell: {df.shape[0]}")
  return df

#############################################################################
def ListCells(params, base_url=BASE_URL, fout=None):
  tags=None; df=pd.DataFrame();
  headers = {"Accept":"application/json", "user_key":params['user_key']}
  url_base = (base_url+'/cells')
  i_chunk=0;
  while True:
    qry = ('{"skip":%d,"limit":%d}'%(i_chunk*N_CHUNK, N_CHUNK))
    url = url_base+('?filter={}'.format(urllib.parse.quote(qry)))
    cells = rest.Utils.GetURL(url, headers=headers, parse_json=True)
    if not cells: break
    for cell in cells:
      if not tags: tags = list(cell.keys())
      df = pd.concat([df, pd.DataFrame({tags[j]:[cell[tags[j]]] for j in range(len(tags))})])
    i_chunk+=1
  if fout: df.to_csv(fout, "\t", index=False)
  logging.info(f"IDs: {len(ids)}")
  logging.info(f"n_cell: {df.shape[0]}")
  return df

#############################################################################
def CountSignatures(params, args, base_url=BASE_URL, fout=None):
  tags=None; df=pd.DataFrame();
  headers = {"Accept":"application/json", "user_key":params['user_key']}
  url = (base_url+'/sigs/count?where='+args.clue_where)
  sigs = rest.Utils.GetURL(url, headers=headers, parse_json=True)
  for sig in sigs:
    if not tags: tags = list(sig.keys())
    df = pd.concat([df, pd.DataFrame({tags[j]:[sig[tags[j]]] for j in range(len(tags))})])
  if fout: df.to_csv(fout, "\t", index=False)
  logging.info(f"IDs: {len(ids)}")
  logging.info(f"n_sig: {df.shape[0]}")
  return df

#############################################################################
def GetSignatures(params, args, base_url=BASE_URL, fout=None):
  tags=None; df=pd.DataFrame();
  fields = ['pert_id','pert_iname','pert_desc','pert_dose','cell_id','provenance_code','target_seq','target_is_lm','target_is_bing','target_zs','dn100_bing','up100_bing']
  headers = {"Accept":"application/json", "user_key":params['user_key']}
  url_base = (base_url+'/sigs')
  i_chunk=0;
  while True:
    qry = ('{"where":%s,"fields":[%s],"skip":%d,"limit":%d}'%(args.clue_where, (','.join(['"%s"'%f for f in fields])), args.skip+i_chunk*N_CHUNK, N_CHUNK))
    url = url_base+('?filter=%s'%(qry))
    sigs = rest.Utils.GetURL(url, headers=headers, parse_json=True)
    if not sigs: break
    logging.debug(json.dumps(sigs, indent=2))
    for sig in sigs:
      if not tags: tags = list(sig.keys())
      df = pd.concat([df, pd.DataFrame({tags[j]:[sig[tags[j]]] for j in range(len(tags))})])
    i_chunk+=1
  if fout: df.to_csv(fout, "\t", index=False)
  logging.info(f"IDs: {len(ids)}")
  logging.info(f"n_sig: {df.shape[0]}")
  return df

#############################################################################



#!/usr/bin/env python3
##############################################################################
### Utility for LincsCloud REST API.
### http://www.lincsproject.org/
### http://api.lincscloud.org/
### http://api.lincscloud.org/a2/usage
### service = cellinfo|geneinfo|instinfo|pertinfo|plateinfo|siginfo
##############################################################################
### "The website lincscloud.org, a Broad Institute developed resource for
### analysis of LINCS Phase 1 (2011-2014) data, will be deprecated in 2015
### as the NIH has recently funded a separate LINCS Data Coordination and
### Integration Center (DCIC)."
##############################################################################
import sys,os,re,json,argparse,time,logging
#
from ..util import rest
#
API_HOST='api.lincscloud.org'
API_BASE_PATH='/a2'
API_KEY="===REPLACE-WITH-KEY==="
#
##############################################################################
def GeneInfo(id_query, base_url, api_key):
  url=base_url+'/geneinfo?q={"pr_gene_symbol":"%s"}'%id_query
  url+='&user_key=%s'%api_key
  logging.debug(url)
  rval=None
  try:
    rval=rest.Utils.GetURL(url, parse_json=True)
  except Exception as e:
    logging.info('HTTP Error (%s): %s'%(res,e))
  return rval

##############################################################################
def PertInfo(id_query,base_url,api_key):
  url=base_url+'/pertinfo?q={"pert_iname":"%s"}'%id_query
  url+='&user_key=%s'%api_key
  logging.debug(url)
  rval=None
  try:
    rval=rest.Utils.GetURL(url,parse_json=True)
  except Exception as e:
    logging.info('HTTP Error (%s): %s'%(res,e))
  return rval

##############################################################################
def PertList_Bioactive(base_url, api_key, fout, nskip=0, nmax=0, nchunk=100):
  n_all=0; n_out=0; n_err=0;
  t0=time.time()
  url=base_url+'/pertinfo?q={"pert_icollection":"BIOA"}'
  url+='&user_key=%s'%api_key
  fields=[
	"pubchem_cid",
	"canonical_smiles",
	"inchi_string",
	"in_summly",
	"molecular_formula",
	"molecular_wt",
	"num_gold",
	"num_inst",
	"num_sig",
	"pert_collection",
	"pert_id",
	"pert_iname",
	"pert_summary",
	"pert_type",
	"pert_vendor"
	]
  fout.write('\t'.join(fields)+'\n')
  done=False
  while True:
    url_this=url+('&sk=%d&l=%d'%(nskip, nchunk))
    logging.debug(url_this)
    rval=rest.Utils.GetURL(url_this, parse_json=True)
    if not rval:
      logging.error('no response: %s'%url)
      break
    perts=rval
    if type(perts) not in (list, tuple):
      logging.debug('perts (%s) = "%s"'%(type(perts),str(perts)))
      nskip+=nchunk
      continue
    for pert in perts:
      if type(pert) is not dict:
        logging.debug('pert (%s) = "%s"'%(type(pert),str(pert)))
        continue
      n_all+=1
      for j,field in enumerate(fields):
        fout.write('%s"%s"'%(('\t' if j>0 else ''),pert[field] if field in pert else ''))
      fout.write('\n')
      n_out+=1
      if nmax>0 and n_out>=nmax:
        done=True
        break
    if done: break
    nskip+=nchunk

  logging.info('n_all: %d  n_out: %d  n_err: %d'%(n_all,n_out,n_err),)
  logging.info('elapsed time: %s'%time.strftime('%Hh:%Mm:%Ss',time.gmtime(time.time()-t0)))

##############################################################################
if __name__=='__main__':
  epilog='''\
LINCS = Library of Integrated Network-based Cellular Signatures,
http://www.lincsproject.org/.
Examples:
"geneinfo?q={'pr_gene_symbol':'TP53'}", 
"pertinfo?q={'pert_iname':'ketorolac'}", 
"pertinfo?q={'pert_icollection':'BIOA'}&c=true", 
"siginfo?q={'sig_id':'CPC019_A549_24H:BRD-K33720404-001-01-9:10'}".
Gene info ID must be gene name.
Perturbagen info ID must be chemical name.
'''
  parser = argparse.ArgumentParser(description='LINCS REST API client', epilog=epilog)
  ops = ['geneinfo', 'pertinfo', 'pertlist_bioactive'] 
  parser.add_argument("op", choices=ops, help='operation')
  parser.add_argument("--i", dest="ifile", help="input file of query IDs")
  parser.add_argument("--o", dest="ofile", help="output (TSV)")
  parser.add_argument("--str_query", help="search substring")
  parser.add_argument("--ids", help="query IDs, comma-separated")
  parser.add_argument("--rex", help="search regex")
  parser.add_argument("--nmax", type=int, default=None, help="max records")
  parser.add_argument("--skip", type=int, default=0, help="skip 1st SKIP queries")
  parser.add_argument("--nchunk", type=int, default=0)
  parser.add_argument("--api_host", default=API_HOST)
  parser.add_argument("--api_base_path", default=API_BASE_PATH)
  parser.add_argument("--api_key", default=API_KEY)
  parser.add_argument("-v", "--verbose", default=0, action="count")
  args = parser.parse_args()

  logging.basicConfig(format='%(levelname)s:%(message)s', level=(logging.DEBUG if args.verbose>1 else logging.INFO))

  base_url='https://'+args.api_host+args.api_base_path

  if args.ifile:
    fin=open(args.ifile)
    if not fin: parser.error('failed to open input file: %s'%args.ifile)
    ids=[]
    while True:
      line=fin.readline()
      if not line: break
      ids.append(line.strip())
  elif args.ids:
    ids = re.split('[,\s]+', args.ids.strip())

  if args.ofile:
    fout=open(args.ofile,"w")
    if not fout: parser.error('cannot open outfile: %s'%args.ofile)
  else:
    fout=sys.stdout

  t0=time.time()

  if args.op == "geneinfo":
    for id_query in ids:
      rval = GeneInfo(id_query, base_url, args.api_key)
      logging.info(json.dumps(rval,indent=2))

  elif args.op == "pertinfo":
    for id_query in ids:
      rval = PertInfo(id_query, base_url, args.api_key)
      logging.info(json.dumps(rval,indent=2))

  elif args.op == "pertlist_bioa":
      PertList_Bioactive(base_url, args.api_key, fout, args.skip, args.max, args.chunk)

  else:
    parser.error('No operation specified.')

  logging.info(('elapsed time: %s'%(time.strftime('%Hh:%Mm:%Ss', time.gmtime(time.time()-t0)))))




#!/usr/bin/env python3
"""
Utility functions for MaayanLab REST APIs.
Alternately could use harmonizomeapi:
http://amp.pharm.mssm.edu/Harmonizome/static/harmonizomeapi.py

https://amp.pharm.mssm.edu/Harmonizome/api/1.0/gene/NANOG
https://amp.pharm.mssm.edu/Harmonizome/api/1.0/gene/NANOG?showAssociations=true
"""
import sys,os,re,json,logging

from ...util import rest
#
API_HOST='amp.pharm.mssm.edu'
API_BASE_PATH='/Harmonizome/api/1.0'
BASE_URL='https://'+API_HOST+API_BASE_PATH
#
##############################################################################
def GetGene(ids, base_url=BASE_URL, fout=None):
  """Gene symbols expected, e.g. NANOG."""
  n_out=0; tags=None;
  for id_this in ids:
    gene = rest.Utils.GetURL(base_url+'/gene/{0}'.format(id_this), parse_json=True)
    logging.debug(json.dumps(gene, indent=2))
    if not tags:
      tags = list(gene.keys())
      fout.write("\t".join(tags)+"\n")
    vals = [(str(gene[tag]) if tag in gene else "") for tag in tags]
    fout.write("\t".join(vals)+"\n")
    n_out+=1
  logging.info("n_out: %d"%(n_out))

##############################################################################
def GetGeneAssociations(ids, base_url=BASE_URL, fout=None):
  """Gene symbols expected, e.g. NANOG."""
  n_out=0; gene_tags=[]; assn_tags=[];
  for id_this in ids:
    gene = rest.Utils.GetURL(base_url+'/gene/{0}?showAssociations=true'.format(id_this), parse_json=True)
    assns = gene["associations"] if "associations" in gene else []
    if not assns: continue
    if not gene_tags:
      for tag in gene.keys():
        if type(gene[tag]) not in (list,dict): gene_tags.append(tag)
    for assn in assns:
      logging.debug(json.dumps(assn, indent=2))
      if not assn_tags:
        assn_tags = list(assn.keys())
        fout.write("\t".join(gene_tags+assn_tags)+"\n")
      vals = [(str(gene[tag]) if tag in gene else "") for tag in gene_tags]+[(str(assn[tag]) if tag in assn else "") for tag in assn_tags]
      fout.write("\t".join(vals)+"\n")
      n_out+=1
  logging.info("n_out: %d"%(n_out))

##############################################################################



#!/usr/bin/env python3
"""
Utility functions for the NLM ChemIDplus REST API.
https://chem.nlm.nih.gov/chemidsearch/api
https://chem.nlm.nih.gov/api/swagger-ui.html
https://chem.nlm.nih.gov/api/v2/api-docs
"""
###
import sys,os,re,argparse,time,logging
#
from .. import chemidplus
#
##############################################################################
if __name__=='__main__':
  ops = ["list_sources", "list_types", "get_id2summary", "get_id2names",
	"get_id2numbers",
	"get_id2toxlist"]
  parser = argparse.ArgumentParser(description="ChemIDPlus REST client")
  parser.add_argument("op", choices=ops,help='OPERATION')
  parser.add_argument("--i", dest="ifile", help="input IDs file")
  parser.add_argument("--ids", help="input IDs (comma-separated)")
  parser.add_argument("--id_type", default="auto", help="input ID type")
  parser.add_argument("--o", dest="ofile", help="output (usually TSV)")
  parser.add_argument("--api_host", default=chemidplus.API_HOST)
  parser.add_argument("--api_base_path", default=chemidplus.API_BASE_PATH)
  parser.add_argument("--skip", type=int, default=0)
  parser.add_argument("--nmax", type=int, default=0)
  parser.add_argument("-v", "--verbose", default=0, action="count")
  args = parser.parse_args()

  logging.basicConfig(format='%(levelname)s:%(message)s', level=(logging.DEBUG if args.verbose>1 else logging.INFO))

  base_url = 'https://'+args.api_host+args.api_base_path

  fout = open(args.ofile, "w") if args.ofile else sys.stdout

  ids=[]
  if args.ifile:
    fin = open(args.ifile)
    while True:
      line = fin.readline()
      if not line: break
      ids.append(line.rstrip())
    fin.close()
  elif args.ids:
    ids = re.split(r'[,\s]+', args.ids)
  logging.info(f"Input IDs: {len(ids)}")

  t0=time.time()

  if args.op == 'list_sources':
    chemidplus.ListSources(base_url, fout)

  elif args.op == 'list_types':
    chemidplus.ListTypes(base_url, fout)

  elif args.op == 'get_id2summary':
    chemidplus.GetId2Summary(ids, args.id_type, base_url, fout)

  elif args.op == 'get_id2names':
    chemidplus.GetId2Names(ids, args.id_type, base_url, fout)

  elif args.op == 'get_id2numbers':
    chemidplus.GetId2Numbers(ids, args.id_type, base_url, fout)

  elif args.op == 'get_id2toxlist':
    chemidplus.GetId2ToxicityList(ids, args.id_type, base_url, fout)

  else:
    parser.error(f"Invalid operation: {args.op}")

  logging.info(('elapsed time: %s'%(time.strftime('%Hh:%Mm:%Ss',time.gmtime(time.time()-t0)))))




#!/usr/bin/env python3
"""
Utility functions for ChEBI SOAP API.
https://www.ebi.ac.uk/chebi/webServices.do
"""
###
import sys,os,re,json,time,urllib.parse,logging,tqdm
import pandas as pd
import requests
import collections
import xmltodict
#
NCHUNK=100
#
API_HOST="www.ebi.ac.uk"
API_BASE_PATH="/webservices/chebi/2.0/test"
BASE_URL="https://"+API_HOST+API_BASE_PATH
#
##############################################################################
def GetEntity(ids, base_url=BASE_URL, fout=None):
  n_out=0; tags=None; df=None;
  for id_this in ids:
    response = requests.get(f"{base_url}/getCompleteEntity?chebiId={id_this}")
    rval_dict = xmltodict.parse(response.content)
    logging.debug(json.dumps(rval_dict, indent=2))
    try:
      result = rval_dict["S:Envelope"]["S:Body"]["getCompleteEntityResponse"]["return"]
    except Exception as e:
      continue
    logging.debug(json.dumps(result, indent=2))
    if not tags: tags = [tag for tag in result.keys() if type(result[tag]) not in (list, dict, collections.OrderedDict)]
    df_this = pd.DataFrame({tags[j]:[result[tags[j]]] for j in range(len(tags))})
    if fout is None: df = pd.concat([df, df_this])
    else: df_this.to_csv(fout, "\t", index=False, header=bool(n_out==0))
    n_out += df_this.shape[0]
  logging.info(f"n_out: {n_out}")
  if fout is None: return df

##############################################################################
def GetEntityChildren(ids, base_url=BASE_URL, fout=None):
  n_out=0; tags=None; df=None;
  for id_this in ids:
    response = requests.get(f"{base_url}/getOntologyChildren?chebiId={id_this}")
    rval_dict = xmltodict.parse(response.content)
    logging.debug(json.dumps(rval_dict, indent=2))
    try:
      result = rval_dict["S:Envelope"]["S:Body"]["getOntologyChildrenResponse"]["return"]
    except Exception as e:
      continue
    logging.debug(json.dumps(result, indent=2))
    children = result["ListElement"] if "ListElement" in result else []
    if type(children) is collections.OrderedDict: children = [children]
    for child in children:
      if not tags: tags = [tag for tag in child.keys() if type(child[tag]) not in (list, dict, collections.OrderedDict)]
      df_this = pd.DataFrame({tags[j]:[child[tags[j]]] for j in range(len(tags))})
      if fout is None: df = pd.concat([df, df_this])
      else: df_this.to_csv(fout, "\t", index=False, header=bool(n_out==0))
      n_out += df_this.shape[0]
  logging.info(f"n_out: {n_out}")
  if fout is None: return df

##############################################################################
def GetEntityParents(ids, base_url=BASE_URL, fout=None):
  n_out=0; tags=None; df=None;
  for id_this in ids:
    response = requests.get(f"{base_url}/getOntologyParents?chebiId={id_this}")
    rval_dict = xmltodict.parse(response.content)
    logging.debug(json.dumps(rval_dict, indent=2))
    try:
      result = rval_dict["S:Envelope"]["S:Body"]["getOntologyParentsResponse"]["return"]
    except Exception as e:
      continue
    logging.debug(json.dumps(result, indent=2))
    parents = result["ListElement"] if "ListElement" in result else []
    if type(parents) is collections.OrderedDict: parents = [parents]
    for parent in parents:
      if not tags: tags = [tag for tag in parent.keys() if type(parent[tag]) not in (list, dict, collections.OrderedDict)]
      df_this = pd.DataFrame({tags[j]:[parent[tags[j]]] for j in range(len(tags))})
      if fout is None: df = pd.concat([df, df_this])
      else: df_this.to_csv(fout, "\t", index=False, header=bool(n_out==0))
      n_out += df_this.shape[0]
  logging.info(f"n_out: {n_out}")
  if fout is None: return df

#############################################################################



#!/usr/bin/env python3
"""
TIGA/TCRD (Target Illumination GWAS Analytics) MySql client utilities
"""
###
import os,sys,re,time,json,logging,yaml,tqdm
import pandas as pd

#############################################################################
def Info(dbcon, fout=None):
  df=None;
  sqls = [
	"SELECT COUNT(DISTINCT study_acc) study_count FROM tiga_provenance",
	"SELECT COUNT(DISTINCT pmid) publication_count FROM tiga_provenance",
	"SELECT COUNT(DISTINCT efoid) trait_count FROM tiga",
	"SELECT COUNT(DISTINCT ensg) gene_count FROM tiga",
	"SELECT COUNT(DISTINCT protein_id) protein_count FROM tiga",
	]
  for sql in sqls:
    df_this = pd.read_sql(sql, dbcon)
    df = pd.concat([df, df_this], axis=1)
  df = df.transpose()
  df.reset_index(drop=False, inplace=True)
  df.columns = ["count", "N"]
  if fout: df.to_csv(fout, "\t", index=False)
  return df

#############################################################################
def ListGenes(dbcon, fout=None):
  df=None;
  sql = """\
SELECT DISTINCT
	tiga.ensg ensemblGeneId,
	tiga.protein_id tcrdProteinId,
	protein.sym,
	protein.description proteinName,
	tiga.geneNtrait,
	tiga.geneNstudy
FROM
	tiga
JOIN
	protein ON protein.id = tiga.protein_id
	;
"""
  df = pd.read_sql(sql, dbcon)
  if fout: df.to_csv(fout, "\t", index=False)
  logging.info(f"n_genes: {df.shape[0]}")
  return df

#############################################################################
def ListTraits(dbcon, fout=None):
  df=None;
  sql = """\
SELECT DISTINCT
	efoid efoId,
	trait efoName,
	traitNgene,
	traitNstudy
FROM
	tiga
	;
"""
  df = pd.read_sql(sql, dbcon)
  if fout: df.to_csv(fout, "\t", index=False)
  logging.info(f"n_traits: {df.shape[0]}")
  return df

#############################################################################
def GetGeneTraitAssociations(geneIds, traitIds, dbcon, fout=None):
  df=None; n_out=0;
  sql = """\
SELECT DISTINCT
	efoid efoId,
	trait efoName,
	traitNgene,
	traitNstudy,
	tiga.ensg ensemblGeneId,
	tiga.protein_id tcrdProteinId,
	protein.sym,
	protein.description proteinName,
	tiga.geneNtrait,
	tiga.geneNstudy,
	tiga.n_study,
	tiga.n_snp,
	tiga.n_snpw,
	tiga.pvalue_mlog_median,
	tiga.or_median,
	tiga.n_beta,
	tiga.study_N_mean,
	tiga.rcras,
	tiga.meanRank,
	tiga.meanRankScore
FROM
	tiga
JOIN
	protein ON protein.id = tiga.protein_id
"""
  if geneIds is None:
    for traitId in traitIds:
      sql_this = sql+f"\nWHERE efoid = '{traitId}'"
      df_this = pd.read_sql(sql_this, dbcon)
      if fout is None:
        df = pd.concat([df, df_this])
      else: df_this.to_csv(fout, "\t", index=False, header=bool(n_out==0))
      n_out += df_this.shape[0]
  elif traitIds is None:
    for geneId in geneIds:
      sql_this = sql+f"\nWHERE ensg = '{geneId}'"
      df_this = pd.read_sql(sql_this, dbcon)
      if fout is None:
        df = pd.concat([df, df_this])
      else: df_this.to_csv(fout, "\t", index=False, header=bool(n_out==0))
      n_out += df_this.shape[0]
  else:
    for geneId in geneIds:
      for traitId in traitIds:
        sql_this = sql+f"\nWHERE efoid = '{traitId}' AND ensg = '{geneId}'"
        df_this = pd.read_sql(sql_this, dbcon)
        if fout is None:
          df = pd.concat([df, df_this])
        else: df_this.to_csv(fout, "\t", index=False, header=bool(n_out==0))
        n_out += df_this.shape[0]
  logging.info(f"n_out: {n_out}")
  if fout is None:
    return df

#############################################################################
def GetGeneTraitProvenance(geneIds, traitIds, dbcon, fout=None):
  df=None; n_out=0;
  sql = """\
SELECT DISTINCT
	tiga_provenance.ensg,
	tiga_provenance.efoid,
	tiga_provenance.study_acc,
	tiga_provenance.pubmedid,
	pubmed.title,
	pubmed.journal,
	pubmed.date,
	pubmed.authors,
	pubmed.abstract
FROM
	tiga_provenance
JOIN
	pubmed ON pubmed.id = tiga_provenance.pubmedid
WHERE
	tiga_provenance.ensg = '{0}' AND tiga_provenance.efoid = '{1}'
	;
"""
  for geneId in geneIds:
    for traitId in traitIds:
      sql_this = sql.format(geneId, traitId)
      df_this = pd.read_sql(sql_this, dbcon)
      if fout is None:
        df = pd.concat([df, df_this])
      else: df_this.to_csv(fout, "\t", index=False, header=bool(n_out==0))
      n_out += df_this.shape[0]
  logging.info(f"n_out: {n_out}")
  if fout is None:
    return df




#!/usr/bin/env python3
"""
LINCS REST API client
New (2019) iLINCS: 
http://www.ilincs.org/ilincs/APIinfo
http://www.ilincs.org/ilincs/APIdocumentation
(http://lincsportal.ccs.miami.edu/dcic/api/ DEPRECATED?)
"""
###
import sys,os,re,json,logging,requests,tqdm
import urllib,urllib.parse
import pandas as pd
#
API_HOST="www.ilincs.org"
API_BASE_PATH="/api"
BASE_URL='https://'+API_HOST+API_BASE_PATH
#
#############################################################################
def GetGene(ids, base_url=BASE_URL, fout=None):
  tags=None; df=None;
  for id_this in ids:
    url = base_url+'/GeneInfos/'+id_this
    response = requests.get(url)
    rval = response.json()
    logging.debug(json.dumps(rval, indent=2))
    if not tags:
      tags = [tag for tag in rval.keys() if type(rval[tag]) not in (list, dict)]
    gene = rval
    df = pd.concat([df, pd.DataFrame({tags[j]:[gene[tags[j]]] for j in range(len(tags))})])
  if fout: df.to_csv(fout, "\t", index=False)
  logging.info(f"IDs: {len(ids)}")
  return df

#############################################################################
def GetDataset(ids, base_url=BASE_URL, fout=None):
  tags=None; df=None;
  for id_this in ids:
    url = base_url+'/PublicDatasets/'+id_this
    response = requests.get(url)
    rval = response.json()
    logging.debug(json.dumps(rval, indent=2))
    if not tags:
      tags = [tag for tag in rval.keys() if type(rval[tag]) not in (list, dict)]
    dset = rval
    df = pd.concat([df, pd.DataFrame({tag:[dset[tag]] for tag in tags})])
  if fout: df.to_csv(fout, "\t", index=False)
  logging.info(f"IDs: {len(ids)}")
  return df

#############################################################################
def GetCompound(ids, base_url=BASE_URL, fout=None):
  n_out=0; tags=None; df=None; tq=None;
  for id_this in ids:
    if tq is None: tq = tqdm.tqdm(total=len(ids), unit="cpds")
    url = f"{base_url}/Compounds/{id_this}"
    response = requests.get(url)
    rval = response.json()
    logging.debug(json.dumps(rval, indent=2))
    cpd = rval
    if not tags:
      tags = list(cpd.keys())
      for tag in tags[:]:
        if type(cpd[tag]) in (list, dict):
          tags.remove(tag)
          logging.info(f"Ignoring tag \"{tag}\"")
    df_this = pd.DataFrame({tag:[compound[tag]] for tag in tags})
    if fout is not None:
      df_this.to_csv(fout, "\t", index=False, header=bool(n_out==0))
    else:
      df = pd.concat([df, df_this])
    n_out+=1
    tq.update(n=1)
  tq.close()
  logging.info(f"IDs: {len(ids)}; n_out: {n_out}")
  return df

#############################################################################
def ListCompounds(base_url=BASE_URL, fout=None):
  n_out=0; tags=None; df=None; tq=None; skip=0; nchunk=100;
  n_total = requests.get(f"{base_url}/Compounds/count").json()['count']
  while True:
    if tq is None: tq = tqdm.tqdm(total=n_total, unit="cpds")
    filter_arg = """%7B"skip"%3A"""+str(skip)+"""%2C"limit"%3A"""+str(nchunk)+"""%7D"""
    #url = f"{base_url}/Compounds?filter={urllib.parse.quote(filter_arg)}"
    url = f"{base_url}/Compounds?filter={filter_arg}"
    response = requests.get(url)
    if response.status_code != requests.codes.ok: break
    rval = response.json()
    if not rval: break
    compounds = rval
    for compound in compounds:
      logging.debug(json.dumps(compound, indent=2))
      if not tags:
        tags = list(compound.keys())
        for tag in tags[:]:
          if type(compound[tag]) in (list, dict):
            tags.remove(tag)
            logging.info(f"Ignoring tag \"{tag}\"")
      df_this = pd.DataFrame({tag:[compound[tag]] for tag in tags})
      if fout is not None: df_this.to_csv(fout, "\t", index=False, header=bool(n_out==0))
      else: df = pd.concat([df, df_this])
      n_out+=1
      tq.update(n=1)
    skip+=nchunk
  tq.close()
  logging.info(f"n_out: {n_out}")
  return df

#############################################################################
def SearchDataset(searchTerm, lincs, base_url=BASE_URL, fout=None):
  n_out=0; tags=None; df=None;
  url = base_url+'/PublicDatasets/findTermMeta'
  d = {'term':searchTerm}
  if lincs: d['lincs'] = True
  response = requests.post(url, data=d)
  rval = response.json()
  logging.debug(json.dumps(rval, indent=2))
  dsets = rval['data'] if 'data' in rval else []
  for dset in dsets:
    logging.debug(json.dumps(dset, indent=2))
    if not tags:
      tags = [tag for tag in dset.keys() if type(dset[tag]) not in (list, dict)]
    df_this = pd.DataFrame({tag:[dset[tag]] for tag in tags})
    if fout is not None: df_this.to_csv(fout, "\t", index=False, header=bool(n_out==0))
    else: df = pd.concat([df, df_this])
    n_out+=1
  logging.info(f"n_out: {n_out}")
  return df

#############################################################################
def SearchSignature(ids, lincs, base_url=BASE_URL, fout=None):
  #SignatureMeta?filter={"where":{"lincspertid":"LSM-2121"},"limit":10}
  n_out=0; tags=None; df=None;
  for id_this in ids:
    url = base_url+'/SignatureMeta?filter={"where":{"lincspertid":"'+id_this+'"}}'
    response = requests.get(url)
    rval = response.json()
    logging.debug(json.dumps(rval, indent=2))
    sigs = rval
    for sig in sigs:
      logging.debug(json.dumps(sig, indent=2))
      if not tags:
        tags = [tag for tag in sig.keys() if type(sig[tag]) not in (list, dict)]
      df_this = pd.DataFrame({tag:[sig[tag]] for tag in tags})
      if fout is not None: df_this.to_csv(fout, "\t", index=False, header=bool(n_out==0))
      else: df = pd.concat([df, df_this])
      n_out+=1
  logging.info(f"IDs: {len(ids)}; n_sig: {df.shape[0]}")
  return df

#############################################################################
def GetSignature(ids, ngene, base_url=BASE_URL, fout=None):
  n_out=0; tags=None; df=None; tq=None;
  url = f"{base_url}/ilincsR/downloadSignature"
  d = {'sigID':(','.join(ids)), 'display':True, 'noOfTopGenes':ngene}
  response = requests.post(url, data=d)
  rval = response.json()
  logging.debug(json.dumps(rval, indent=2))
  genes = rval['data']['signature'] if 'data' in rval and 'signature' in rval['data'] else []
  for gene in genes:
    logging.debug(json.dumps(gene, indent=2))
    if not tags:
      tags = [tag for tag in gene.keys() if type(gene[tag]) not in (list, dict)]
    df_this = pd.concat([df, pd.DataFrame({tag:[gene[tag]] for tag in tags})])
    if fout is not None: df_this.to_csv(fout, "\t", index=False, header=bool(n_out==0))
    else: df = pd.concat([df, df_this])
    n_out+=1
  logging.info(f"IDs: {len(ids)}; n_out: {n_out}")
  return df

#############################################################################



#!/usr/bin/env python3
"""
Access to Ensembl biomart REST API.
https://m.ensembl.org/info/data/biomart/biomart_restful.html
"""
import sys,os,re,time,json,logging,tqdm,io
import pandas as pd
import requests, urllib.parse
#
API_HOST='www.ensembl.org'
API_BASE_PATH='/biomart/martservice'
#
BASE_URL='http://'+API_HOST+API_BASE_PATH
#
ENSG2NCBI_xml="""\
<?xml version="1.0" encoding="UTF-8"?>
<!DOCTYPE Query>
<Query  virtualSchemaName = "default" formatter = "TSV" header = "1" uniqueRows = "0" count = "" datasetConfigVersion = "0.6" >

	<Dataset name = "hsapiens_gene_ensembl" interface = "default" >
		<Attribute name = "ensembl_gene_id" />
		<Attribute name = "ensembl_gene_id_version" />
		<Attribute name = "entrezgene_id" />
	</Dataset>
</Query>
"""
#
ENSG2HGNC_xml="""\
<?xml version="1.0" encoding="UTF-8"?>
<!DOCTYPE Query>
<Query  virtualSchemaName = "default" formatter = "TSV" header = "1" uniqueRows = "0" count = "" datasetConfigVersion = "0.6" >

	<Dataset name = "hsapiens_gene_ensembl" interface = "default" >
		<Attribute name = "ensembl_gene_id" />
		<Attribute name = "ensembl_gene_id_version" />
		<Attribute name = "hgnc_id" />
		<Attribute name = "hgnc_symbol" />
	</Dataset>
</Query>
"""
#
ENSG2NCBIHGNC_xml="""\
<?xml version="1.0" encoding="UTF-8"?>
<!DOCTYPE Query>
<Query  virtualSchemaName = "default" formatter = "TSV" header = "1" uniqueRows = "0" count = "" datasetConfigVersion = "0.6" >

	<Dataset name = "hsapiens_gene_ensembl" interface = "default" >
		<Attribute name = "ensembl_gene_id" />
		<Attribute name = "ensembl_gene_id_version" />
		<Attribute name = "entrezgene_id" />
		<Attribute name = "hgnc_id" />
		<Attribute name = "hgnc_symbol" />
	</Dataset>
</Query>
"""
#
##############################################################################
def XMLQuery(xmltxt, base_url=BASE_URL, fout=None):
  url_this = base_url+f"?query={urllib.parse.quote(xmltxt)}"
  logging.debug(url_this)
  rval = requests.get(url_this)
  if not rval.ok:
    logging.error(f"{rval.status_code}")
    logging.debug(rval.text)
    return None
  df = pd.read_table(io.StringIO(rval.text), "\t")
  if fout: df.to_csv(fout, "\t", index=False)
  logging.info(f"Output rows: {df.shape[0]}; cols: {df.shape[1]} ({str(df.columns.tolist())})")
  return df

##############################################################################
def ENSG2NCBI(base_url=BASE_URL, fout=None):
  return XMLQuery(ENSG2NCBI_xml, base_url, fout)

##############################################################################
def ENSG2HGNC(base_url=BASE_URL, fout=None):
  return XMLQuery(ENSG2HGNC_xml, base_url, fout)

##############################################################################
def ENSG2NCBIHGNC(base_url=BASE_URL, fout=None):
  return XMLQuery(ENSG2NCBIHGNC_xml, base_url, fout)

##############################################################################



#!/usr/bin/env python3
###
import sys,os,time,json,argparse,re,logging
#
from ..util import rest
#
#############################################################################
def GetAnnotations(base_url, mode, pmids, fout):
  n_assn=0; n_hit=0;
  fout.write('sourcedb\tsourceid\tbegin\tend\tobj_type\tobj\n')
  for pmid in pmids:
    url = base_url+'/%s/%s/JSON'%(mode, pmid)
    rval = rest.GetURL(url, parse_json=True)
    if not rval:
      logging.info('not found: %s'%(pmid))
      continue

    n_assn_this=0
    sources = rval if type(rval) is list else [rval]
    for source in sources:
      sourceDb = source['sourcedb'] if 'sourcedb' in source else ''
      sourceId = source['sourceid'] if 'sourceid' in source else ''
      anns = source['denotations'] if (type(source) is dict and 'denotations') in source else []

      for ann in anns:
        obj = ann['obj'] if 'obj' in ann else None
        begin = ann['span']['begin'] if 'span' in ann and 'begin' in ann['span'] else ''
        end = ann['span']['end'] if 'span' in ann and 'end' in ann['span'] else ''
        if obj and begin and end:
          obj_type,obj_id = re.split(':', obj, 1)
          fout.write('%s\t%s\t%d\t%d\t%s\t%s\n'%(sourceDb, sourceId, begin, end, obj_type, obj_id))
          n_assn_this+=1
    if n_assn_this: n_hit+=1
    n_assn+=n_assn_this

  logging.info('n_in = %d (PMIDs)'%(len(pmids)))
  logging.info('n_hit = %d (PMIDs with associations)'%(n_hit))
  logging.info('n_miss = %d (PMIDs with NO associations)'%(len(pmids)-n_hit))
  logging.info('n_assn = %d (total associations)'%(n_assn))

#############################################################################



#!/usr/bin/env python
#############################################################################
### pubchem_assay_fetch.py - from input AIDs, fetch full dataset
#############################################################################
import os,sys,re,time,getopt,gzip,zipfile

from ... import pubchem

PROG=os.path.basename(sys.argv[0])

ASSAY_DATA_DIR='/home/data/pubchem/bioassay/csv/data'

#############################################################################
def ErrorExit(msg):
  print >>sys.stderr,msg
  sys.exit(1)

#############################################################################
def MergeListIntoHash(hsh,lst):
  for x in lst:
    if not hsh.has_key(x):
      hsh[x]=1
    else:
      hsh[x]+=1
  return

#############################################################################
def ExtractSIDs(fpath_csv_gz):
  try:
    f=gzip.open(fpath_csv_gz)
  except:
    print >>sys.stderr, 'ERROR: could not open %s'%(fpath)
    return []
  ftxt=f.read()
  f.close()
  sids_this=pubchem.ftp.Utils.ExtractOutcomes(ftxt,None,False)
  return sids_this

#############################################################################
if __name__=='__main__':
  usage='''
  %(PROG)s - fetch assay csvs from AIDs (from local mirror)

  required:
  --i AIDFILE .......... AIDs (CSV w/ AID 1st ok)
  or
  --aids AIDS .......... list of AIDs (comma separated)
  and
  --odir ODIR .......... dir for output files

  options:
  --o OFILE ............ SIDs extracted from assay CSVs
  --keep_dirtree ....... output in directory tree (as in PubChem FTP)
  --v .................. verbose
  --h .................. this help
'''%{'PROG':PROG}
  ifile=None; odir=None; verbose=0; ofile=None; aidslist=None; keep_dirtree=False;
  opts,pargs = getopt.getopt(sys.argv[1:],'',['h','v','vv','odir=','i=','o=','aids=','keep_dirtree'])
  if not opts: ErrorExit(usage)
  for (opt,val) in opts:
    if opt=='--h': ErrorExit(usage)
    elif opt=='--i': ifile=val
    elif opt=='--odir': odir=val
    elif opt=='--out_sids': ofile=val
    elif opt=='--aids': aidslist=val
    elif opt=='--keep_dirtree': keep_dirtree=True
    elif opt=='--vv': verbose=2
    elif opt=='--v': verbose=1
    else: ErrorExit('Illegal option: %s'%val)

  if not odir:
    ErrorExit('-odir required\n'+usage)

  if not os.access(ASSAY_DATA_DIR,os.R_OK):
    ErrorExit('cannot find: %s'%ASSAY_DATA_DIR)

  aids=[];
  if aidslist:
    aids=map(lambda x:int(x),(re.split(r'[\s,]',aidslist.strip())))
  elif ifile:
    faids=file(ifile)
    if not faids:
      ErrorExit('cannot open: %s\n%s'%(ifile,usage))
    i=0;
    while True:
      line=faids.readline()
      if not line: break
      line=line.strip()
      if not line: continue
      i+=1
      try:
        field=re.sub('[,\s].*$','',line)   ## may be addl field[s]
        aid=int(field)
        aids.append(aid)
      except:
        print >>sys.stderr,'cannot parse aid: "%s"'%line
        continue
    print >>sys.stderr,'aids read: %d'%len(aids)
  else:
    ErrorExit('-in or -aids required\n'+usage)

  aids.sort()

  sids={}

  t0=time.time()
  n_out=0; n_not_found=0;
  for aid in aids:
    if verbose:
      print >>sys.stderr, '%d:\t'%(aid),
    is_found=False
    for fname_zip in os.listdir(ASSAY_DATA_DIR):
      if not re.search('\.zip',fname_zip): continue
      aid_from=re.sub(r'^([\d]+)_([\d]+)\.zip$',r'\1',fname_zip)
      aid_to=re.sub(r'^([\d]+)_([\d]+)\.zip$',r'\2',fname_zip)
      try:
        aid_from=int(aid_from)
        aid_to=int(aid_to)
      except:
        print >>sys.stderr, 'ERROR: cannot parse AIDs from fname_zip: "%s"'%fname_zip
        continue
      if aid<aid_from or aid>aid_to:
        continue
      if verbose:
        print >>sys.stderr, '(%s)'%(fname_zip),
      fpath_zip=ASSAY_DATA_DIR+'/'+fname_zip
      try:
        zf=zipfile.ZipFile(fpath_zip,'r')
      except:
        print >>sys.stderr, 'ERROR: cannot read fpath_zip: "%s"'%fpath_zip
        continue
      flist_csv_gz=zf.namelist()
      zf.close()
      for fpath_csv_gz in flist_csv_gz:
        aid_this=None
        if not re.search('\.csv\.gz',fpath_csv_gz): continue
        try:
          if re.search(r'/',fpath_csv_gz):
            txt=re.sub(r'^.*/(\d*)\.csv\.gz',r'\1',fpath_csv_gz)
          else:
            txt=re.sub(r'\.csv\.gz','',fpath_csv_gz)
          aid_this=int(txt)
        except:
          print >>sys.stderr, 'cannot parse AID: "%s"'%fpath_csv_gz
          continue
        if aid==aid_this:
          zf=zipfile.ZipFile(fpath_zip,'r')
          cwd=os.getcwd()
          os.chdir(odir)
          zf.extract(fpath_csv_gz)
          if not keep_dirtree:
            d,f = os.path.split(fpath_csv_gz)
            os.rename(fpath_csv_gz,f)
          n_out+=1
          is_found=True
          os.chdir(cwd)
          zf.close()
          if ofile:
            MergeListIntoHash(sids,ExtractSIDs(odir+"/"+fpath_csv_gz))
          break
    if not is_found:
      n_not_found+=1
    if verbose:
      print >>sys.stderr, '\t[%s]'%(is_found)

  if ofile:
    fout_sids=open(ofile,"w+")
    if not fout_sids:
      print >>sys.stderr, 'cannot open: %s'%ofile
    else:
      for sid in sids.keys():
        fout_sids.write("%d\n"%sid)
      fout_sids.close()

  print >>sys.stderr, '%s: input AIDs: %d'%(PROG,len(aids))
  print >>sys.stderr, '%s: output assay csv datafiles: %d'%(PROG,n_out)
  print >>sys.stderr, '%s: assays not found: %d'%(PROG,n_not_found)
  if ofile:
    print >>sys.stderr, '%s: output SIDs: %d'%(PROG,len(sids.keys()))
  print >>sys.stderr, ('%s: total elapsed time: %s'%(PROG,time.strftime('%Hh:%Mm:%Ss',time.gmtime(time.time()-t0))))



#!/usr/bin/env python
'''
Retrieve compounds by SID or CID from PubChem/PUG (circa 2010).
'''
import sys,os,re,urllib,time,getopt,tempfile
import pug_utils

PROG=os.path.basename(sys.argv[0])

PUGURL='http://pubchem.ncbi.nlm.nih.gov/pug/pug.cgi'
POLL_WAIT=10
MAX_WAIT=600

#############################################################################
def QueryPug_fetchmols(dbname,ids,fmt='smiles',gzip=False,ntries=20,verbose=0):
  if gzip: compres='gzip'
  else: compres='none'
  print >>sys.stderr, 'DEBUG: fmt = "%s"'%fmt
  qxml='''\
<PCT-Data><PCT-Data_input><PCT-InputData>
  <PCT-InputData_download><PCT-Download><PCT-Download_uids>
    <PCT-QueryUids><PCT-QueryUids_ids>
    <PCT-ID-List><PCT-ID-List_db>%(DBNAME)s</PCT-ID-List_db><PCT-ID-List_uids>
'''%{'DBNAME':dbname}
  for id in ids:
    qxml+=('<PCT-ID-List_uids_E>%d</PCT-ID-List_uids_E>\n'%id)
  qxml+='''\
        </PCT-ID-List_uids></PCT-ID-List>
      </PCT-QueryUids_ids></PCT-QueryUids>
    </PCT-Download_uids>
    <PCT-Download_format value="%(FORMAT)s" />
    <PCT-Download_compression value="%(COMPRES)s" />
  </PCT-Download></PCT-InputData_download>
</PCT-InputData></PCT-Data_input></PCT-Data>
'''%{'FORMAT':fmt.lower(),'COMPRES':compres}
  if verbose:
    print >>sys.stderr,'connecting %s...'%PUGURL
    print >>sys.stderr,'submitting query for %d compounds...'%len(ids)
  f=pug_utils.UrlOpenTry(PUGURL,qxml,ntries)
  pugxml=f.read()
  f.close()
  status,reqid,url,qkey,wenv,error=pug_utils.ParsePugXml(pugxml)
  return status,reqid,url,error

#############################################################################
def ReverseFields(ofile):
  '''Fix smiles file from PubChem.'''
  fin=file(ofile)
  fd,ftmp=tempfile.mkstemp('.smi',PROG)
  while True:
    line=fin.readline()
    if not line: break
    os.write(fd,line)
  fin.close()
  os.close(fd)
  fout=file(ofile,"w")
  fin=file(ftmp)
  i=0
  while True:
    line=fin.readline()
    if not line: break
    i+=1
    line=line.rstrip()
    if not line: break
    fields=re.split('\s',line)
    if len(fields)==1:
      smiles='*'
      id=fields[0]
      print >>sys.stderr, 'NOTE: no structure: (line = %d) %s'%(i,line)
    else:
      smiles=fields[1]
      id=fields[0]
    fout.write("%s %s\n"%(smiles,id))
  fout.close()
  fin.close()
  os.unlink(ftmp)

#############################################################################
if __name__=='__main__':
  idtype='SID';
  USAGE='''
  %(PROG)s - fetch PubChem molecule files (SD or SMI) for IDs

  required:
  --i INFILE .......... input IDs file
  --o OUTFILE ......... output molecule file

  options:
  --idtype CID|SID .... [default=%(IDTYPE)s]
  --fmt FMT ........... smiles|sdf [default via filename]
  --nmax N ............ maximum N IDs
  --skip N ............ skip N IDs
  --n_chunk N ......... IDs per PUG request [10000]
  --ftp_ntries N ...... max tries per ftp-get
  --gz ................ output gzipped [default via filename]
  --v[v] .............. verbose [very]
  --h ................. this help
'''%{'PROG':PROG,'IDTYPE':idtype}

  ifile=None; ofile=None; verbose=0; skip=0; nmax=0; ftp_ntries=20;
  fmt=None; gzip=False; n_chunk=10000;
  opts,pargs = getopt.getopt(sys.argv[1:],'',['h','v','vv','i=','ftp_ntries=',
  'o=','fmt=','gz','skip=','nmax=','n_chunk=',
  'idtype='])
  if not opts: pug_utils.ErrorExit(USAGE)
  for (opt,val) in opts:
    if opt=='--h': pug_utils.ErrorExit(USAGE)
    elif opt=='--i': ifile=val
    elif opt=='--o': ofile=val
    elif opt=='--nmax': nmax=int(val)
    elif opt=='--idtype': idtype=val
    elif opt=='--n_chunk': n_chunk=int(val)
    elif opt=='--ftp_ntries': ftp_ntries=int(val)
    elif opt=='--fmt': fmt=val
    elif opt=='--gz': gzip=True
    elif opt=='--skip': skip=int(val)
    elif opt=='--vv': verbose=2
    elif opt=='--v': verbose=1
    else: pug_utils.ErrorExit('Illegal option: %s'%val)

  if not (ifile and ofile):
    pug_utils.ErrorExit('input and output files required\n'+USAGE)
  if (ofile[-3:]=='.gz'):
    gzip=True
    ext=re.sub('^.*\.','',ofile[:-3])
  else:
    ext=re.sub('^.*\.','',ofile)
  if not fmt: fmt=ext
  if fmt=='smi': fmt='smiles'
  if fmt.lower() not in ('smiles','sdf'):
    pug_utils.ErrorExit('format "smiles" or "sdf" required\n'+USAGE)
  if idtype.upper() not in ('SID','CID'):
    pug_utils.ErrorExit('idtype "SID" or "CID" required\n'+USAGE)

  fids=file(ifile)
  if not fids:
    pug_utils.ErrorExit('cannot open: %s\n%s'%(ifile,USAGE))
  fout=file(ofile,'w')
  if not fout:
    pug_utils.ErrorExit('cannot open: %s\n%s'%(ofile,USAGE))
  dbname='pccompound' if idtype.upper()=='CID' else 'pcsubstance'

  ids=[]; n=0;
  while True:
    line=fids.readline()
    if not line: break
    line=line.strip()
    if not line: continue
    n+=1
    if skip and n<=skip: continue
    if nmax and n>nmax: break
    try:
      field=re.sub('\s.*$','',line)	## may be addl field
      id=int(field)
      ids.append(id)
    except:
      print >>sys.stderr,'cannot parse id: "%s"'%line
      continue
  print >>sys.stderr,'ids read: %d'%len(ids)

  n=0; i_query=0; nbytes_total=0; n_query_total=0;
  while n<len(ids):
    i_query+=1
    n_query=len(ids[n:n+n_chunk])
    print >>sys.stderr,'Q%d. [%d-%d] of %d'%(i_query,n+1,n+n_query,len(ids))
    status,reqid,url,error=QueryPug_fetchmols(dbname,ids[n:n+n_chunk],fmt,gzip,ftp_ntries,verbose)
    pug_utils.CheckStatus(status,error)
    if verbose:
      print >>sys.stderr,'query submit status: %s'%status
    if not reqid and not url:
      pug_utils.ErrorExit('query submit ok but no ID; quitting.')
    i_poll=0
    while not url:
      if i_poll*POLL_WAIT>MAX_WAIT:
        pug_utils.PollPug(reqid,'cancel',verbose)
        pug_utils.ErrorExit('max wait exceeded (%d sec); quitting.'%MAX_WAIT)
      print >>sys.stderr,'polling PUG [ID=%d]...'%reqid
      status,reqid,url,qkey,wenv,error = pug_utils.PollPug(reqid,'status',verbose)
      pug_utils.CheckStatus(status,error)
      if url: break
      if verbose:
        print >>sys.stderr,'%d sec wait (status=%s)...'%(POLL_WAIT,status)
      time.sleep(POLL_WAIT)
      i_poll+=1
    if verbose:
      print >>sys.stderr,'URL: %s'%url
      print >>sys.stderr,'downloading to %s...'%ofile
    nbytes=pug_utils.DownloadUrl(url,fout)
    nbytes_total+=nbytes;
    print >>sys.stderr,'%ss: %d (%.2fMB)'%(dbname,n_query,nbytes/1e6)
    n_query_total+=n_query
    n+=n_chunk
  fout.close()
  if fmt.lower()=='smiles': ReverseFields(ofile)

  print >>sys.stderr,'%s: total %ss: %d (%.2fMB)'%(PROG,dbname,n_query_total,nbytes_total/1e6)
  sys.stderr.write('%s: format: %s'%(PROG,fmt))
  if gzip: sys.stderr.write('(gzipped)')
  sys.stderr.write('\n')



#!/usr/bin/env python
#############################################################################
### pubchem_ftp_assay_results.py - input AID, SIDs, output CSV outcomes.
#############################################################################
import os,sys,re,time,getopt,gzip,zipfile,tempfile,shutil

from ... import pubchem

PROG=os.path.basename(sys.argv[0])

DATADIR='/home/data/pubchem/bioassay/csv/data'

#############################################################################
if __name__=='__main__':
  def ErrorExit(msg):
    print >>sys.stderr,msg
    sys.exit(1)

  usage='''
  %(PROG)s - assay csvs from AIDs (from local mirror)

  required:
  --aid AID ............ AID
  and
  --i SIDFILE .......... SID file
  or
  --sids SIDS .......... SIDs, comma-separated

  options:
  --o OFILE ............ assay results CSV
  --datadir DATADIR .... [%(DATADIR)s]
  --v .................. verbose
  --h .................. this help
'''%{'PROG':PROG,
	'DATADIR':DATADIR
	}
  ifile=None;
  aid=None;
  verbose=0;
  ofile=None;
  opts,pargs = getopt.getopt(sys.argv[1:],'',['h','v','vv','i=','o=',
	'aid=','sids='])
  if not opts: ErrorExit(usage)
  for (opt,val) in opts:
    if opt=='--h': ErrorExit(usage)
    elif opt=='--i': ifile=val
    elif opt=='--o': ofile=val
    elif opt=='--aid': aid=int(val)
    elif opt=='--sids': sids=set(map(lambda x:int(x),(re.split(r'[\s,]',val.strip()))))
    elif opt=='--vv': verbose=2
    elif opt=='--v': verbose=1
    else: ErrorExit('Illegal option: %s'%val)

  if not os.access(DATADIR,os.R_OK):
    ErrorExit('cannot find: %s'%DATADIR)

  if ifile:
    sids=set()
    fsids=file(ifile)
    if not fsids:
      ErrorExit('cannot open: %s\n%s'%(ifile,usage))
    i=0;
    while True:
      line=fsids.readline()
      if not line: break
      line=line.strip()
      if not line: continue
      i+=1
      try:
        field=re.sub('[,\s].*$','',line)   ## may be addl field[s]
        sid=int(field)
        sids.add(sid)
      except:
        print >>sys.stderr,'cannot parse sid: "%s"'%line
        continue
    print >>sys.stderr,'sids read: %d'%len(sids)
  elif not sids:
    ErrorExit('-i or -sids required\n'+usage)

  if ofile:
    fout=open(ofile,"w+")
  else:
    fout=sys.stdout

  t0=time.time()

  tmpdir=tempfile.mkdtemp(prefix='pubchem_ftp',suffix='')

  n_out=0; n_not_found=0;

  if verbose:
    print >>sys.stderr, '%d:'%(aid),
  is_found=False
  for fname_zip in os.listdir(DATADIR):
    if not re.search('\.zip',fname_zip): continue
    aid_from=re.sub(r'^([\d]+)_([\d]+)\.zip$',r'\1',fname_zip)
    aid_to=re.sub(r'^([\d]+)_([\d]+)\.zip$',r'\2',fname_zip)
    try:
      aid_from=int(aid_from)
      aid_to=int(aid_to)
    except:
      print >>sys.stderr, 'ERROR: cannot parse AIDs from fname_zip: "%s"'%fname_zip
      continue
    if aid<aid_from or aid>aid_to:
      continue
    if verbose:
      print >>sys.stderr, '(%s)'%(fname_zip)
    fpath_zip=DATADIR+'/'+fname_zip
    try:
      zf=zipfile.ZipFile(fpath_zip,'r')
    except:
      print >>sys.stderr, 'ERROR: cannot read fpath_zip: "%s"'%fpath_zip
      continue
    flist_csv_gz=zf.namelist()
    zf.close()
    for fpath_csv_gz in flist_csv_gz:
      aid_this=None
      if not re.search('\.csv\.gz',fpath_csv_gz): continue
      try:
        if re.search(r'/',fpath_csv_gz):
          txt=re.sub(r'^.*/(\d*)\.csv\.gz',r'\1',fpath_csv_gz)
        else:
          txt=re.sub(r'\.csv\.gz','',fpath_csv_gz)
        aid_this=int(txt)
      except:
        print >>sys.stderr, 'cannot parse AID: "%s"'%fpath_csv_gz
        continue
      if aid==aid_this:
        zf=zipfile.ZipFile(fpath_zip,'r')
        cwd=os.getcwd()
        os.chdir(tmpdir)
        zf.extract(fpath_csv_gz)
        n_out+=1
        is_found=True
        os.chdir(cwd)
        zf.close()
        fpath=(tmpdir+"/"+fpath_csv_gz)
        #print >>sys.stderr, 'DEBUG: %s'%(fpath)
        try:
          fin=gzip.open(fpath)
        except:
          print >>sys.stderr, 'ERROR: could not open %s'%(fpath)
        pubchem.ftp.Utils.ExtractResults(fin,aid,sids,fout,verbose)
        fin.close()
        break
    if verbose and not is_found:
      print >>sys.stderr, ''

  fout.close()

  shutil.rmtree(tmpdir)




#!/usr/bin/env python3
"""
http://data.bioontology.org/documentation
The National Center for Biomedical Ontology was founded as one of the
National Centers for Biomedical Computing, supported by the NHGRI, the
NHLBI, and the NIH Common Fund.
"""
###
import sys,os,argparse,re,yaml,logging,time
#
from .. import ncbo
from ..util import yaml as util_yaml
#
#############################################################################
if __name__=='__main__':
  EPILOG="""The National Center for Biomedical Ontology was founded as one of the National Centers for Biomedical Computing, supported by the NHGRI, the NHLBI, and the NIH Common Fund."""
  parser = argparse.ArgumentParser(description='NCBO REST API client utility', epilog=EPILOG)
  OPS = ['recommendOntologies']
  parser.add_argument("op", choices=OPS, help="OPERATION")
  parser.add_argument("--i", dest="ifile", help="input texts")
  parser.add_argument("--o", dest="ofile", help="output (TSV)")
  parser.add_argument("--text", help="input text")
  parser.add_argument("--api_host", default=ncbo.API_HOST)
  parser.add_argument("--api_base_path", default=ncbo.API_BASE_PATH)
  parser.add_argument("--param_file", default=os.environ["HOME"]+"/.ncbo.yaml")
  parser.add_argument("--api_key", help="API key")
  parser.add_argument("-v", "--verbose", default=0, action="count")

  args = parser.parse_args()

  logging.basicConfig(format="%(levelname)s:%(message)s", level=(logging.DEBUG if args.verbose>1 else logging.INFO))

  base_url = "https://"+args.api_host+args.api_base_path

  fout = open(args.ofile, "w+") if args.ofile else sys.stdout

  params = util_yaml.ReadParamFile(args.param_file)
  if args.api_key: params["API_KEY"] = args.api_key
  if not params["API_KEY"]:
    parser.error("Please specify valid API_KEY via --api_key or --param_file") 

  texts=[];
  if args.ifile:
    with open(args.ifile) as fin:
      while True:
        line = fin.readline()
        if not line: break
        texts.append(line.rstrip())
    logging.info(f"input texts: {len(texts)}")
  elif args.text:
    texts = [args.text]

  t0 = time.time()

  if args.op == "recommendOntologies":
    ncbo.RecommendOntologies(base_url, params["API_KEY"], texts, fout)

  else:
    parser.error(f"Invalid operation: {args.op}")

  logging.info(("Elapsed time: %s"%(time.strftime('%Hh:%Mm:%Ss',time.gmtime(time.time()-t0)))))



#!/usr/bin/env python3
###
import sys,os,json,re,logging,tqdm
import pandas as pd
#
from ..util import rest
#
API_HOST="icite.od.nih.gov"
API_BASE_PATH="/api/pubs"
BASE_URL='https://'+API_HOST+API_BASE_PATH
#
NCHUNK=100;
#
#############################################################################
def GetStats(pmids, base_url=BASE_URL, fout=None):
  """Request multiply by chunk. Lists of PMIDs (e.g. references,
cited_by) reported as counts."""
  n_in=0; n_out=0; tags=None; df=pd.DataFrame(); tq=None;
  quiet = bool(logging.getLogger().getEffectiveLevel()>15)
  while True:
    if tq is None and not quiet: tq = tqdm.tqdm(total=len(pmids), unit="pmids")
    if n_in>=len(pmids): break
    pmids_this = pmids[n_in:n_in+NCHUNK]
    n_in += (NCHUNK if n_in+NCHUNK < len(pmids) else len(pmids)-n_in)
    url_this = (f"""{base_url}?pmids={(','.join(pmids_this))}""")
    rval = rest.GetURL(url_this, parse_json=True)
    if not rval: break
    logging.debug(json.dumps(rval, indent=2))
    url_self = rval['links']['self']
    pubs = rval['data']
    for pub in pubs:
      if not tags: tags = list(pub.keys())
      df_this = pd.DataFrame({tags[j]:[pub[tags[j]]] for j in range(len(tags))})
      if fout: df_this.to_csv(fout, "\t", index=False, header=bool(n_out==0))
      df = pd.concat([df, df_this])
      n_out+=1
    if not quiet:
      for j in range(len(pmids_this)): tq.update()
  logging.info(f"n_in: {len(pmids)}; n_out: {n_out}")
  return df

#############################################################################
def GetStats_single(pmids, base_url=BASE_URL, fout=None):
  """Request singly."""
  tags=None; df=pd.DataFrame(); tq=None;
  for pmid in pmids:
    if tq is None: tq = tqdm.tqdm(total=len(pmids), unit="pmids")
    tq.update()
    url = base_url+'/'+pmid
    pub = rest.GetURL(url, parse_json=True)
    if not pub:
      logging.info(f"Not found: {pmid}")
      continue
    if not tags: tags = list(pub.keys())
    df = pd.concat([df, pd.DataFrame({tags[j]:[pub[tags[j]]] for j in range(len(tags))})])
  if fout: df.to_csv(fout, "\t", index=False)
  logging.info(f"n_in: {len(pmids)}; n_out: {df.shape[0]}")
  return df

#############################################################################



#!/usr/bin/env python3
#############################################################################
### CDC  REST API client
### https://tools.cdc.gov/api/docs/info.aspx
### https://tools.cdc.gov/api/v2/resources
#############################################################################
import sys,os,re,json,csv,logging
import urllib.parse,requests
#
from ..util import rest
#
#############################################################################
def ListResources(base_url, resource, fout):
  tags=None; n_rsc=0; offset=0; nchunk=100;
  url=base_url+'/%s'%resource
  while True:
    url_this='%s?offset=%d&max=%d'%(url, offset, nchunk)
    rval=rest.GetURL(url_this, parse_json=True)
    try:
      rscs = rval['results']
      pag = rval['meta']['pagination']
      count = pag['count']
      total = pag['total']
      nextUrl = pag['nextUrl']
    except:
      break
    for rsc in rscs:
      if not tags:
        tags=rsc.keys()
        fout.write('\t'.join(tags)+'\n')
      vals = [str(rsc[tag]) if tag in rsc else '' for tag in tags]
      fout.write('\t'.join(vals)+'\n')
      n_rsc+=1
    if count>=total or not nextUrl:
      break
    else:
      offset+=nchunk
  logging.info('n_rsc = %d (total %s)'%(n_rsc, resource))

#############################################################################



#!/usr/bin/env python3
"""
See http://www.guidetopharmacology.org/webServices.jsp
"""
import sys,os,re,argparse,time,logging,urllib.parse,json
#
from .. import iuphar
#
API_HOST='www.guidetopharmacology.org'
API_BASE_PATH='/services'
#
#############################################################################
if __name__=='__main__':
  SEARCH_TYPE='substructure'; 
  epilog='''\
operations:
  list_targets: list targets (selected by type, db, accessionId);
  list_target_families: list target families ;
  get_target: get target (requires ID);
  get_target_interactions: get target interactions (requires ID);
  get_target_dblinks: get target dblinks (requires ID);
  get_target_substrates: get target substrates (requires ID);
  get_target_function: get target function (requires ID);
  get_target_products: get target products (requires ID);
  get_target_gpinfo: get target gene protein info (requires ID);
  get_ligand_synonyms: get ligand synonyms (requires ID);
  list_ligands: list ligands (selected by type, db, accessionId);
  get_ligand: get ligand (requires ID);
  get_ligand_interactions: get ligand interactions (requires ID);
  get_ligand_dblinks: get ligand dblinks (requires ID);
  get_ligand_structure: get ligand structure (requires ID);
  get_target_synonyms: get target synonyms (requires ID);
  search_ligand: structure search (requires smiles query);
'''

  parser = argparse.ArgumentParser(description='IUPHAR/GuideToPharmacolog API client', epilog=epilog)
  ops=[ "list_targets", "list_target_families", "get_target", "get_target_interactions", "get_target_dblinks", "get_target_substrates", "get_target_function", "get_target_products", "get_target_gpinfo", "get_ligand_synonyms", "list_ligands", "get_ligand", "get_ligand_interactions", "get_ligand_dblinks", "get_ligand_structure", "get_target_synonyms", "search_ligand" ]
  parser.add_argument("op", choices=ops, help='operation')
  parser.add_argument("--i", dest="ifile", help="input file of IDs (targetId|ligandId|accessionId)")
  parser.add_argument("--ids", help="ID list (comma-separated)(targetId|ligandId|accessionId)")
  parser.add_argument("--o", dest="ofile", help="output file (TSV)")
  parser.add_argument("--tgt_type", help="GPCR|NHR|LGIC|VGIC|OtherIC|Enzyme|CatalyticReceptor|Transporter|OtherProtein|AccessoryProtein")
  parser.add_argument("--lig_type", help="Synthetic organic|Metabolite|Natural product|Endogenous peptide|Peptide|Antibody|Inorganic|Approved|Withdrawn|Labelled|INN")
  parser.add_argument("--itr_type", help="Activator|Agonist|Allosteric modulator|Antagonist|Antibody|Channel blocker|Gating inhibitor|Inhibitor|Subunit-specific")
  parser.add_argument("--aff", type=float, help="affinity minimun")
  parser.add_argument("--aff_type", help="pA2|pEC50|pIC50|pKB|pKd|pKi")
  parser.add_argument("--smiles", help="structure search query")
  parser.add_argument("--search_type", help="substructure|exact|similarity")
  parser.add_argument("--species", help="Human|Mouse|Rat|etc.")
  parser.add_argument("--db", help="external accessionId database ChEMBL|EntrezGene|HGNC|PubChemCID|UniProt|etc.")
  parser.add_argument("--api_host", default=API_HOST)
  parser.add_argument("--api_base_path", default=API_BASE_PATH)
  parser.add_argument("-v", "--verbose", default=0, action="count")

  args = parser.parse_args()

  logging.basicConfig(format='%(levelname)s:%(message)s', level=(logging.DEBUG if args.verbose>1 else logging.INFO))

  BASE_URL = 'https://'+args.api_host+args.api_base_path

  if args.ofile:
    fout = open(args.ofile, "w+")
  else:
    fout = sys.stdout

  ids=[]
  if args.ifile:
    fin = open(args.ifile)
    while True:
      line = fin.readline()
      if not line: break
      ids.append(line.strip())
  elif args.ids:
    ids = re.split(r'[,\s]+', args.ids.strip())

  t0=time.time()

  if args.op == "list_targets":
    iuphar.ListTargets(BASE_URL, args.tgt_type, args.db, ids, fout)

  elif args.op == "list_target_families":
    iuphar.ListTargetFamilies(BASE_URL, args.tgt_type, fout)

  elif args.op == "get_target":
    iuphar.GetTarget(BASE_URL, ids, fout)

  elif args.op == "get_target_interactions":
    iuphar.GetInteractions(BASE_URL, 'targets', ids, args.species, args.itr_type, args.aff_type, args.aff, fout)

  elif args.op == "get_target_dblinks":
    iuphar.GetDblinks(BASE_URL, 'targets', ids, args.species, args.db, fout)

  elif args.op == "get_target_substrates":
    iuphar.GetTargetSubstrates(BASE_URL, ids, args.species, fout)

  elif args.op == "get_target_products":
    iuphar.GetTargetProducts(BASE_URL, ids, args.species, fout)

  elif args.op == "get_target_function":
    iuphar.GetTargetFunction(BASE_URL, ids, args.species, fout)

  elif args.op == "get_target_gpinfo":
    iuphar.GetTargetGeneProteinInfo(BASE_URL, ids, fout)

  elif args.op == "get_target_synonyms":
    iuphar.GetSynonyms(BASE_URL, 'targets', ids, fout)

  elif args.op == "list_ligands":
    iuphar.ListLigands(BASE_URL, args.lig_type, args.db, ids, fout)

  elif args.op == "get_ligand":
    iuphar.GetLigand(BASE_URL, ids, fout)

  elif args.op == "get_ligand_interactions":
    iuphar.GetInteractions(BASE_URL, 'ligands', ids, args.species, args.itr_type, args.aff_type, args.aff, fout)

  elif args.op == "get_ligand_dblinks":
    iuphar.GetDblinks(BASE_URL, 'ligands', ids, fout)

  elif args.op == "get_ligand_structure":
    iuphar.GetLigandStructure(BASE_URL, ids, fout)

  elif args.op == "get_ligand_synonyms":
    iuphar.GetSynonyms(BASE_URL, 'ligands', ids, fout)

  elif args.op == "search_ligand":
    if not (args.smiles and args.search_type):
      parser.error('--smiles and --search_type required.'+usage)
    iuphar.SearchLigand(BASE_URL, args.smiles, args.search_type, fout)

  else:
    parser.error('Invalid operation: %s'%args.op)
      
  logging.info('Elapsed time: %s'%(time.strftime('%Hh:%Mm:%Ss',time.gmtime(time.time()-t0))))




#!/usr/bin/env python
'''
Search for text in assay descriptions (XML)

Jeremy J Yang
22 Oct 2014
'''
import os,sys,re,time,getopt,gzip,zipfile
import xml.dom.minidom

from ..util import xml_utils

PROG=os.path.basename(sys.argv[0])

#ASSAY_DESC_DIR='/home/data/pubchem/bioassay/xml/description'
ASSAY_DESC_DIR='/home/data/pubchem/bioassay/csv/description'

SCRATCHDIR='/tmp/'+PROG+'_SCRATCHDIR'

#############################################################################
def ParseAssayXml(assayxml):
  aname,ameth,adesc,atargs=None,None,None,None
  dom=xml.dom.minidom.parseString(assayxml)
  tag='PC-AssayDescription_name'
  anames=xml_utils.DOM_GetLeafValsByTagName(dom,tag)
  if anames: aname=anames[0]
  tag='PC-AssayDescription_activity-outcome-method'
  ameths=xml_utils.DOM_GetAttr(dom,tag,'value')
  if ameths: ameth=ameths[0]
  tag='PC-AssayDescription_description_E'
  adescs=xml_utils.DOM_GetLeafValsByTagName(dom,tag)
  if adescs: adesc=('\n'.join(adescs))

  tag='PC-AssayTargetInfo_name'
  atargs_names=xml_utils.DOM_GetLeafValsByTagName(dom,tag)
  tag='PC-AssayTargetInfo_molecule-type'
  atargs_types=xml_utils.DOM_GetAttr(dom,tag,'value')
  tag='PC-AssayTargetInfo_mol-id'
  atargs_ids=xml_utils.DOM_GetLeafValsByTagName(dom,tag)
  gids=[]
  if atargs_names:
    atargs=''
    for i,atargs_name in enumerate(atargs_names):
      if i>0: atargs+=';  '
      atargs+=('%d. %s'%(i+1,atargs_name))
      if len(atargs_ids)>i and len(atargs_types)>i:
        atargs+=(' [%s:%s]'%(atargs_types[i],atargs_ids[i]))
      if atargs_types[i]=='protein':
        gids.append(int(atargs_ids[i]))

  return aname,ameth,adesc,atargs,gids

#############################################################################
def ErrorExit(msg):
  print >>sys.stderr,msg
  sys.exit(1)

#############################################################################
if __name__=='__main__':
  usage='''
  %(PROG)s - search assay names and descriptions

  required:
  --string=<STRING>   ... search substring
  or
  --regexp=<REGEXP>   ... search regular expression
  or
  --out_all_aid_file=<out_all_aid_file> ... csv file: aid,name,method,target[s]

  options:
  --desc           ... search descriptions (default only names)
  --casesensitive  ... case sensitive
  --out_aids=<outfile> ... output file of matched AIDs
  --out_all_aid_file=<out_all_aid_file> ... csv file: aid,name,method,target[s]
  --out_all_gid_file=<outfile> ... output file of all protein GIDs
  --v            ... verbose
  --vv           ... very verbose
  --h            ... this help
'''%{'PROG':PROG}
  string=None; regexp=None; out_aids_file=None; out_all_aid_file=None;
  out_all_gid_file=None;
  casesensitive=False; desc=False; verbose=0;
  opts,pargs = getopt.getopt(sys.argv[1:],'',['h','v','vv','string=',
  'out_all_aid_file=', 'out_all_gid_file=',
  'regexp=','out_aids=','casesensitive','desc'])
  if not opts: ErrorExit(usage)
  for (opt,val) in opts:
    if opt=='--h': ErrorExit(usage)
    elif opt=='--string': string=val
    elif opt=='--regexp': regexp=val
    elif opt=='--out_aids': out_aids_file=val
    elif opt=='--out_all_aid_file': out_all_aid_file=val
    elif opt=='--out_all_gid_file': out_all_gid_file=val
    elif opt=='--casesensitive': casesensitive=True
    elif opt=='--desc': desc=True
    elif opt=='--vv': verbose=2
    elif opt=='--v': verbose=1
    else: ErrorExit('Illegal option: %s'%val)

  if not (string or regexp or out_all_aid_file):
    ErrorExit('-string or -regexp or -out_all_aid_file required\n'+usage)

  fout_aid=None
  if out_aids_file:
    fout_aid=open(out_aids_file,'w')
    if not fout_aid:
      ErrorExit('cannot open: %s'%out_aids_file)

  fout_all_aid=None
  if out_all_aid_file:
    fout_all_aid=open(out_all_aid_file,'w')
    if not fout_all_aid:
      ErrorExit('cannot open: %s'%out_all_aid_file)
    fout_all_aid.write('AID,name,method,target')
    if verbose>1: fout_all_aid.write('description')
    fout_all_aid.write('\n')
  fout_all_gid=None
  if out_all_gid_file:
    fout_all_gid=open(out_all_gid_file,'w')
    if not fout_all_gid:
      ErrorExit('cannot open: %s'%out_all_gid_file)

  if not os.access(ASSAY_DESC_DIR,os.R_OK):
    ErrorExit('cannot find: %s'%ASSAY_DESC_DIR)

# files=[]
# for file in os.listdir(ASSAY_DESC_DIR):
#   if not re.search('\.descr\.xml\.gz',file): continue
#   try:
#     aid=int(re.sub('\.descr\.xml\.gz','',file))
#   except:
#     print >>sys.stderr, 'cannot parse AID: "%s"'%file
#     continue
#   fpath=ASSAY_DESC_DIR+'/'+file
#   files.append((fpath,aid))

  if string and not regexp: regexp=string
  if regexp:
    if casesensitive:
      rob=re.compile(regexp)
    else:
      rob=re.compile(regexp,re.I)

  t0=time.time()

  aids={}
  badfiles=[]
  n_gids_out=0

# for fpath,aid in files:
  files=[]
  for fname_zip in os.listdir(ASSAY_DESC_DIR):
    if not re.search('\.zip',fname_zip): continue
    fpath_zip=ASSAY_DESC_DIR+'/'+fname_zip
    try:
      zf=zipfile.ZipFile(fpath_zip,'r')
    except:
      print >>sys.stderr, 'ERROR: cannot read fpath_zip: "%s"'%fpath_zip
      continue
    flist_xml_gz=zf.namelist()
    zf.close()
    for fpath_xml_gz in flist_xml_gz:
      if not re.search('\.descr\.xml\.gz',fpath_xml_gz): continue
      try:
        if re.search(r'/',fpath_xml_gz):
          txt=re.sub(r'^.*/(\d*)\.descr\.xml\.gz',r'\1',fpath_xml_gz)
        else:
          txt=re.sub(r'\.descr\.xml\.gz','',fpath_xml_gz)
        aid=int(txt)
      except:
        print >>sys.stderr, 'cannot parse AID: "%s"'%fpath_xml_gz
        print >>sys.stderr, 'DEBUG txt: "%s"'%txt
        continue
      files.append((fpath_zip,fpath_xml_gz,aid))

  if not os.access(SCRATCHDIR,os.R_OK):
    try:
      os.mkdir(SCRATCHDIR)
    except:
      print >>sys.stderr, 'ERROR: failed to create SCRATCHDIR %s'%SCRATCHDIR
      sys.exit(1)

  for f in files:
    fpath_zip,fpath_xml_gz,aid = f
    if verbose>1: print >>sys.stderr, '%d: %s:%s'%(aid,fpath_zip,fpath_xml_gz)
    zf=zipfile.ZipFile(fpath_zip,'r')
    cwd=os.getcwd()
    os.chdir(SCRATCHDIR)
    zf.extract(fpath_xml_gz)
    os.chdir(cwd)
    zf.close()
    f_xml=gzip.open(SCRATCHDIR+'/'+fpath_xml_gz)
    fxml=f_xml.read()
    f_xml.close()
    if not fxml:
      print >>sys.stderr, 'ERROR: file empty: AID %d: %s'%(aid,fpath)
      badfiles.append(fpath)
      continue

    aname,ameth,adesc,atargs,gids = ParseAssayXml(fxml)
    if fout_all_aid:
      fout_all_aid.write('%d,"%s","%s","%s"'%(aid,aname,ameth,atargs))
      if verbose>1:
        fout_all_aid.write(',"%s"'%(adesc))
      fout_all_aid.write('\n')

    if fout_all_gid:
      for gid in gids:
        fout_all_gid.write('%d\n'%(gid))
        n_gids_out+=1

    ok=False
    if regexp:
      if rob.search(aname):
        if verbose:
          print >>sys.stderr, 'name match: AID %d: %s'%(aid,aname)
        ok=True
      elif desc and rob.search(adesc):
        if verbose: print >>sys.stderr, 'desc match: AID %d: %s'%(aid,adesc)
        ok=True
    if ok:
      aids[aid]=(aname,adesc)
      if fout_aid:
        fout_aid.write('%d\n'%aid)
    os.unlink(SCRATCHDIR+'/'+fpath_xml_gz)

  if fout_aid: fout_aid.close()
  if fout_all_aid: fout_all_aid.close()
  if fout_all_gid: fout_all_gid.close()

  for dir in os.listdir(SCRATCHDIR):
    subdir=SCRATCHDIR+"/"+dir
    for f in os.listdir(subdir):
      os.unlink(subdir+'/'+f)
    os.rmdir(subdir)
  os.rmdir(SCRATCHDIR)

  print >>sys.stderr, '%s: total assays: %d'%(PROG,len(files))
  print >>sys.stderr, '%s: matched assays: %d'%(PROG,len(aids.keys()))
  if n_gids_out:
    print >>sys.stderr, '%s: target protein GIDs written: %d'%(PROG,n_gids_out)
  print >>sys.stderr, '%s: ERRORS: bad files: %d'%(PROG,len(badfiles))
  for badfile in badfiles:
    print >>sys.stderr, '\tbadfile: %s'%(os.path.basename(badfile))
  print >>sys.stderr, ('%s: total elapsed time: %s'%(PROG,time.strftime('%Hh:%Mm:%Ss',time.gmtime(time.time()-t0))))



import os,sys,unittest

from .. import drugcentral

class TestAPI(unittest.TestCase):

  def __init__(self, methodName=""):
    super().__init__(methodName)
    self.params = drugcentral.ReadParamFile(os.environ['HOME']+"/.drugcentral.yaml")
    self.dbcon = drugcentral.Utils.Connect(self.params['DBHOST'], self.params['DBPORT'], self.params['DBNAME'], self.params['DBUSR'], self.params['DBPW'])

  def test_Version(self):
    self.assertTrue(type(drugcentral.Version(self.dbcon)) is not None)

  def test_Version_02(self):
    self.assertEqual(drugcentral.Version(self.dbcon).shape[0], 1)

  def test_Describe(self):
    self.assertTrue(drugcentral.Describe(self.dbcon).shape[0]>10)

  def test_Counts(self):
    self.assertTrue(drugcentral.Counts(self.dbcon).shape[0]>10)

  def test_ListStructures(self):
    df = drugcentral.ListStructures(self.dbcon)
    self.assertTrue(df.shape[0]>4000)

  def test_ListStructures2Smiles(self):
    df = drugcentral.ListStructures2Smiles(self.dbcon)
    self.assertTrue(df.shape[0]>4000)

  def test_ListProducts(self):
    df = drugcentral.ListProducts(self.dbcon)
    self.assertTrue(df.shape[0]>4000)

  def test_ListActiveIngredients(self):
    df = drugcentral.ListActiveIngredients(self.dbcon)
    self.assertTrue(df.shape[0]>4000)

  def test_ListIndications(self):
    self.assertTrue(drugcentral.ListIndications(self.dbcon).shape[0]>100)

  def test_SearchIndications(self):
    self.assertTrue(drugcentral.SearchIndications(self.dbcon, "Alzheimer").shape[0]>0)

  def test_GetStructure(self):
    self.assertTrue(drugcentral.GetStructure(self.dbcon, ["1725"]).shape[0]==1)

  def test_GetStructureBySynonym(self):
    self.assertTrue(drugcentral.GetStructureBySynonym(self.dbcon, ["zantac"]).shape[0]==1)

  def test_GetStructureIds(self):
    self.assertTrue(drugcentral.GetStructureIds(self.dbcon, ["1725"]).shape[0]>5)

  def test_GetStructureProducts(self):
    self.assertTrue(drugcentral.GetStructureProducts(self.dbcon, ["1725"]).shape[0]>10)

#############################################################################
if __name__ == '__main__':
  unittest.main(verbosity=2)



#!/usr/bin/env python3
"""
Utility functions for JensenLab REST APIs.
https://api.jensenlab.org/Textmining?type1=-26&id1=DOID:10652&type2=9606&limit=10&format=json
https://api.jensenlab.org/Textmining?query=jetlag[tiab]%20OR%20jet-lag[tiab]&type2=9606&limit=10&format=json
https://api.jensenlab.org/Knowledge?type1=-26&id1=DOID:10652&type2=9606&limit=10&format=json
https://api.jensenlab.org/Experiments?type1=-26&id1=DOID:10652&type2=9606&limit=10&format=json
"""
import sys,os,re,json,time,logging
import pandas as pd

from ..util import rest
#
API_HOST='api.jensenlab.org'
API_BASE_PATH=''
BASE_URL='https://'+API_HOST+API_BASE_PATH
#
##############################################################################
def GetDiseaseGenes(channel, ids, nmax, base_url=BASE_URL, fout=None):
  tags=[]; df=pd.DataFrame();
  for id_this in ids:
    rval = rest.GetURL(base_url+f'/{channel}?type1=-26&id1={id_this}&type2=9606&limit={nmax}&format=json', parse_json=True)
    genes = rval[0] #dict
    ensgs = list(genes.keys())
    flag = rval[1] #?
    for ensg in ensgs:
      gene = genes[ensg]
      logging.debug(json.dumps(gene, indent=2))
      if not tags: tags = list(gene.keys())
      df = pd.concat([df, pd.DataFrame({tags[j]:[gene[tags[j]]] for j in range(len(tags))})])
  if fout: df.to_csv(fout, "\t", index=False)
  logging.info("n_out: {}".format(df.shape[0]))
  return df

##############################################################################
def GetPubmedComentionGenes(ids, base_url=BASE_URL, fout=None):
  """Search by co-mentioned terms."""
  tags=[]; df=pd.DataFrame();
  for id_this in ids:
    rval = rest.GetURL(base_url+f'/Textmining?query={id_this}[tiab]&type2=9606&limit=10&format=json', parse_json=True)
    genes = rval[0] #dict
    ensgs = list(genes.keys())
    flag = rval[1] #?
    for ensg in ensgs:
      gene = genes[ensg]
      logging.debug(json.dumps(gene, indent=2))
      if not tags: tags = list(gene.keys())
      df = pd.concat([df, pd.DataFrame({tags[j]:[gene[tags[j]]] for j in range(len(tags))})])
  if fout: df.to_csv(fout, "\t", index=False)
  logging.info("n_out: {}".format(df.shape[0]))
  return df

##############################################################################



#!/usr/bin/env python3
###
import sys,os,json,argparse,logging

from ...util import graphql as util_graphql

#GQL_HOST="ncats-ifx.appspot.com"
GQL_HOST="pharos-api.ncats.io"
GQL_BASE_PATH="/graphql"
GQL_URL="https://"+GQL_HOST+GQL_BASE_PATH

#############################################################################
def DemoGql():
  return """\
{
  search(term: "gout") {
    diseaseResult {
      count
      facets {
        facet
        values {
          name
          value
        }
      }
      diseases {
        name
        associationCount
        associations {
          did
          name
          type
        }
      }
    }
  }
}
"""

#############################################################################
if __name__ == "__main__":
  parser = argparse.ArgumentParser(description='Pharos GraphQL client utility')
  OPS = ['query', 'demo', 'getSchema']
  parser.add_argument("op", choices=OPS, help='OPERATION')
  parser.add_argument("--i", dest="ifile", help="input file, GraphQL")
  parser.add_argument("--graphql", help="input GraphQL")
  parser.add_argument("--o", dest="ofile", help="output (TSV)")
  parser.add_argument("--gql_host", default=GQL_HOST)
  parser.add_argument("--gql_base_path", default=GQL_BASE_PATH)
  parser.add_argument("-v", "--verbose", default=0, action="count")
  args = parser.parse_args()

  logging.basicConfig(format='%(levelname)s:%(message)s', level=(logging.DEBUG if args.verbose>1 else logging.INFO))

  gql_url='https://'+args.gql_host+args.gql_base_path

  fout = open(args.ofile) if args.ofile else sys.stdout

  graphql=None;
  if args.ifile:
    with open(args.ifile) as fin:
      graphql = fin.read()
  elif args.graphql:
    graphql = args.graphql
  logging.debug(graphql)

  if args.op == 'query':
    util_graphql.RunQuery(gql_url, graphql, fout)

  elif args.op == 'demo':
    util_graphql.RunQuery(gql_url, DemoGql(), fout)

  elif args.op == 'getSchema':
    parser.error(f"Unimplemented operation: {args.op}")

  else:
    parser.error(f"Invalid operation: {args.op}")




#!/usr/bin/env python3
"""
MeSH XML utility functions.

 MeSH XML
 Download: https://www.nlm.nih.gov/mesh/download_mesh.html
 Doc: https://www.nlm.nih.gov/mesh/xml_data_elements.html
  
 <DescriptorRecord DescriptorClass="1">
 1 = Topical Descriptor.
 2 = Publication Types, for example, 'Review'.
 3 = Check Tag, e.g., 'Male' (no tree number)
 4 = Geographic Descriptor (Z category of tree number).
  
 Category "C" : Diseases
 Category "F" : Psychiatry and Psychology
 Category "F03" : Mental Disorders
 Thus, include "C*" and "F03*" only.
 Terms can have multiple TreeNumbers; diseases can be in non-disease cateories, in addition to a disease category.
"""
###
import sys,os,re,argparse,logging

from .. import mesh

BRANCHES={
	'A':'Anatomy',
	'B':'Organisms',
	'C':'Diseases',
	'D':'Chemicals and Drugs',
	'E':'Analytical, Diagnostic and Therapeutic Techniques, and Equipment',
	'F':'Psychiatry and Psychology',
	'G':'Phenomena and Processes',
	'H':'Disciplines and Occupations',
	'I':'Anthropology, Education, Sociology, and Social Phenomena',
	'J':'Technology, Industry, and Agriculture',
	'K':'Humanities',
	'L':'Information Science',
	'M':'Named Groups',
	'N':'Health Care',
	'V':'Publication Characteristics',
	'Z':'Geographicals'}

#############################################################################
if __name__=='__main__':
  BRANCH='C'
  EPILOG = f"""
operations:
desc2csv: descriptors XML input;
supp2csv: supplementary records XML input;
Branches:
	{"; ".join([f"{k}: {BRANCHES[k]}" for k in sorted(BRANCHES.keys())])}
"""
  parser = argparse.ArgumentParser(description='MeSH XML utility', epilog=EPILOG)
  ops=['desc2csv', 'supp2csv']
  parser.add_argument("op", choices=ops, help='operation')
  parser.add_argument("--i", dest="ifile", help="input MeSH XML file [stdin]")
  parser.add_argument("--o", dest="ofile", help="output file (TSV)")
  parser.add_argument("--branch", choices=BRANCHES, default=BRANCH, help="top-level branch of MeSH tree")
  parser.add_argument("--force", action="store_true", help="ignore UTF-8 encoding errors")
  parser.add_argument("-v", "--verbose", default=0, action="count")
  args = parser.parse_args()

  logging.basicConfig(format='%(levelname)s:%(message)s', level=(logging.DEBUG if args.verbose>1 else logging.INFO))

  fin = open(args.ifile, "r") if args.ifile else sys.stdin

  fout = open(args.ofile, "w") if args.ofile else sys.stdout

  if args.op == "desc2csv":
    mesh.Desc2Csv(args.branch, fin, fout)

  elif args.op == "supp2csv":
    mesh.Supp2Csv(args.branch, fin, fout)

  else:
    parser.error(f"Invalid operation: {args.op}")



#!/usr/bin/env python3
#############################################################################
### CDC  REST API client
### https://tools.cdc.gov/api/docs/info.aspx
### https://tools.cdc.gov/api/v2/resources
#############################################################################
import sys,os,argparse,re,time,logging
#
from .. import cdc
#
#############################################################################
if __name__=='__main__':
  API_HOST="tools.cdc.gov"
  API_BASE_PATH="/api/v2/resources"
  ops = ["list_sources", "list_topics", "list_organizations", "list_audiences"]
  parser = argparse.ArgumentParser( description='CDC REST API client utility')
  parser.add_argument("op", choices=ops, help='OPERATION (select one)')
  parser.add_argument("--i", dest="ifile", help="input file")
  parser.add_argument("--nmax", help="list: max to return")
  parser.add_argument("--o", dest="ofile", help="output (TSV)")
  parser.add_argument("--api_host", default=API_HOST)
  parser.add_argument("--api_base_path", default=API_BASE_PATH)
  parser.add_argument("-v", "--verbose", action="count", default=0)
  args = parser.parse_args()

  logging.basicConfig(format='%(levelname)s:%(message)s', level=(logging.DEBUG if args.verbose>1 else logging.INFO))

  api_base_url='https://'+args.api_host+args.api_base_path

  fout = open(args.ofile, "w+") if args.ofile else sys.stdout

  if args.op=="list_sources":
    cdc.ListResources(api_base_url, 'sources', fout)

  elif args.op=="list_topics":
    cdc.ListResources(api_base_url, 'topics', fout)

  elif args.op=="list_organizations":
    cdc.ListResources(api_base_url, 'organizations', fout)

  elif args.op=="list_audiences":
    cdc.ListResources(api_base_url, 'audiences', fout)

  else:
    parser.error('Operation invalid: {}'.format(args.op))



#!/usr/bin/env python3
"""
utility functions for SLAP, and utility application.

See: http://slapfordrugtargetprediction.wikispaces.com/API

URL form
http://cheminfov.informatics.indiana.edu/rest/Chem2Bio2RDF/slap/5591:PPARG
"""
###
import sys,os,re,argparse,time,logging
import urllib,urllib.parse,json,csv

from xml.etree import ElementTree
from xml.parsers import expat

from ...util import rest
#
#############################################################################
### Scrape HTML for predicted-target table for CSV
### 
### all targets have Chem2Bio2RDF links like:
### http://chem2bio2rdf.org/uniprot/resource/gene/AMPD1
### "approved interaction" targets have Chem2Bio2RDF links like:
### http://cheminfov.informatics.indiana.edu/rest/Chem2Bio2RDF/cid_gene/2995:ADRA1D
### all targets have SLAP links like:
### http://cheminfov.informatics.indiana.edu:8080/slap/slap.jsp?cid=2995&gene=NFKB2
#############################################################################
def Drug2Targets(base_url,cids,fout):
  n_cid=0; n_found=0; n_not_found=0; n_none_predicted=0;
  fout.write('"cid","target","p_value","score","type","chemohub"\n')
  for cid in cids:
    n_cid+=1
    logging.info('%9s: '%(cid))
    try:
      rval=rest.Utils.GetURL(base_url+'/cid=%s'%(cid))
    except Exception as e:
      logging.error('Exception: %s'%(e))
      n_not_found+=1
      continue
    htm=rval
    if not htm:
      n_not_found+=1
      logging.info('%9s: CID not found.'%(cid))
      continue
    elif re.search('No target predicted', htm):
      n_none_predicted+=1
      logging.info('%9s: No targets predicted.'%(cid))
      continue
    n_found+=1
    htm = re.sub(r'[\n\r]', '', htm)
    htm = re.sub(r'</table>.*$', '', htm)
    htm = re.sub(r'^.*<table[^>]*>', '', htm)
    rows = re.split('<tr>', htm)
    logging.debug('n_rows = %d'%len(rows))
    logging.info('%9s: Targets predicted: %d'%(cid,len(rows)-1))
    for row in rows:
      if not row: continue
      row = re.sub('</?tr>', '', row, re.I)
      row = re.sub('^<td>(.*)</td>$', '\\1', row, re.I)
      vals_raw = re.split(r'</td><td>', row)
      if not vals_raw: continue
      logging.debug('vals_raw[0] = "%s"'%vals_raw[0])
      if vals_raw[0] == 'target':
        continue #kludge
      vals=[];
      for val in vals_raw:
        val = re.sub('</?t[dr]>', '', val, re.I)
        url = None
        if re.search(r'<a href="([^"]+)">', val):
          url = re.sub(r'^.*<a href="([^"]+)">.*$', '\\1', val, re.I)
          val = re.sub(r'^.*<a href="[^"]+">([^<]*)</a>.*$', '\\1', val, re.I)
          val = '%s [%s]'%(val,url) if vals else val
        vals.append(val)
      vals = [cid]+vals
      vals = [csv_utils.ToStringForCSV(val) for val in vals]
      fout.write(','.join(vals)+'\n')

  logging.info('n_cid = %d'%(n_cid))
  logging.info('n_found = %d'%(n_found))
  logging.info('n_not_found = %d'%(n_not_found))
  logging.info('n_none_predicted = %d'%(n_none_predicted))

#############################################################################
def Drug2BioSimilarDrugs(base_url,cids,fout):
  n_cid=0; n_found=0; n_not_found=0; n_none_predicted=0;
  fout.write('"query_cid","pubchem_cid","drug_name","similarity","related_diseases","atc"\n')
  for cid in cids:
    n_cid+=1
    logging.info('%9s: '%(cid))
    try:
      rval=rest.Utils.GetURL(base_url+'/cid=%s'%(cid))
    except Exception as e:
      logging.error('Exception: %s'%(e))
      n_not_found+=1
      continue
    htm=rval
    if not htm:
      n_not_found+=1
      logging.info('%9s: CID not found.'%(cid))
      continue
    elif re.search('No target predicted', htm): #FIX
      n_none_predicted+=1
      logging.info('%9s: No bio-similar drugs found.'%(cid))
      continue
    n_found+=1
    htm = re.sub(r'[\n\r]', '', htm)
    htm = re.sub(r'^.*Biological Similar Drugs', '', htm)
    htm = re.sub(r'</table>.*$', '', htm)
    htm = re.sub(r'^.*<table[^>]*>', '', htm)
    rows = re.split('<tr>', htm)
    logging.debug('n_rows = %d'%len(rows))
    logging.info('%9s: Bio-similar drugs predicted: %d'%(cid,len(rows)-1))
    for row in rows:
      if not row: continue
      row = re.sub('</?tr>', '', row, re.I)
      row = re.sub('^<td>(.*)</td>$', '\\1', row, re.I)
      vals_raw = re.split(r'</td><td>', row)
      if not vals_raw: continue
      if vals_raw[0] == 'PubChem CID':
        continue #kludge
      vals=[];
      for val in vals_raw:
        val = re.sub('</?t[dr]>', '', val, re.I)
        url = None
        if re.match('<img .*>$', val, re.I): continue #skip imgs
        if re.search(r'<a href="([^"]+)">', val):
          url = re.sub(r'^.*<a href="([^"]+)">.*$', '\\1', val, re.I)
          val = re.sub(r'^.*<a href="[^"]+">([^<]*)</a>.*$', '\\1', val, re.I)
          val = '%s [%s]'%(val,url) if vals else val
        vals.append(val)
      vals = [cid]+vals
      vals = [csv_utils.ToStringForCSV(val) for val in vals]
      fout.write(','.join(vals)+'\n')

  logging.info('n_cid = %d'%(n_cid))
  logging.info('n_found = %d'%(n_found))
  logging.info('n_not_found = %d'%(n_not_found))
  logging.info('n_none_predicted = %d'%(n_none_predicted))


#############################################################################
def Target2Drugs(base_url,tids,fout):
  n_tid=0; n_found=0; n_not_found=0; n_none_assn=0;
  fout.write('"target","cid","sources","paths","smiles"\n')
  for tid in tids:
    n_tid+=1
    logging.info('%9s: '%(tid))
    try:
      rval=rest.Utils.GetURL(base_url+'/target=%s'%(tid))
    except Exception as e:
      logging.error('Exception: %s'%(e))
      n_not_found+=1
      continue
    htm=rval
    logging.debug('htm = "%s"'%(htm))
    if not htm:
      n_not_found+=1
      logging.info('%9s: target not found.'%(tid))
      continue
    elif re.search('No compounds predicted', htm): #FIX
      n_none_assn+=1
      logging.info('%9s: No compounds associated.'%(tid))
      continue

    n_found+=1
    htm = re.sub(r'[\n\r]', '', htm)
    htm = re.sub(r'</table>.*$', '', htm)
    htm = re.sub(r'^.*<table[^>]*>', '', htm)
    rows = re.split('<tr>', htm)
    logging.debug('n_rows = %d'%len(rows))
    logging.info('%9s: Compounds associated: %d'%(tid,len(rows)-1))
    for row in rows:
      if not row: continue
      row = re.sub('</?tr>', '', row, re.I)
      row = re.sub('^<td>(.*)</td>$', '\\1', row, re.I)
      vals_raw = re.split(r'</td><td>', row)
      if not vals_raw: continue
      logging.debug('vals_raw[0] = "%s"'%vals_raw[0])
      if vals_raw[0] == 'cid':
        continue #kludge
      vals=[];
      for j,val in enumerate(vals_raw):
        val = re.sub('</?t[dr]>', '', val, re.I)
        if j==2:
          val = re.sub(' +', '; ', val.strip()) #sources
        if re.match('<img .*>$', val, re.I): continue #skip imgs
        url = None
        if re.search(r'<a href="([^"]+)">', val):
          url = re.sub(r'^.*<a href="([^"]+)">.*$', '\\1', val, re.I)
          val = re.sub(r'^.*<a href="[^"]+">([^<]*)</a>.*$', '\\1', val, re.I)
          val = '%s [%s]'%(val,url) if vals else val
        vals.append(val)
      vals = [tid]+vals
      vals = [csv_utils.ToStringForCSV(val) for val in vals]
      fout.write(','.join(vals)+'\n')

  logging.info('n_tid = %d'%(n_tid))
  logging.info('n_found = %d'%(n_found))
  logging.info('n_not_found = %d'%(n_not_found))
  logging.info('n_none_assn = %d'%(n_none_assn))

#############################################################################
### Given input drug (CID) and target (TID), find paths, subnetwork, and
### SLAP association scores and type.
#############################################################################
###	<b><i>Input: </i></b> Compound: 5591; Target: PPARG
###	<b><i>P value: </i></b>9.06524962196e-06 (very strong)
###	<b><i>Type: </i></b><a
###	href="http://cheminfov.informatics.indiana.edu/rest/Chem2Bio2RDF/cid_gene/5591:PPARG">approved binding</a>
###	*Smaller p value, stronger association
###	<graphml>
###	...
###	<b><i>Input: </i></b> Compound: 176870; Target: TRIP13
###	<b><i>P value: </i></b>0.8795 (undecided)
###	<b><i>Type: </i></b>predicted
###	*Smaller p value, stronger association
#############################################################################
### Output CSV file contains IDs and scores only.
### If odir and path exists, write full graphml file with IDs in name.
#############################################################################
def DrugTargetPaths(base_url, cids, tids, cid_skip, cid_nmax, tid_skip, tid_nmax, odir, fout):
  errs={}; score_types={}; score_notes={};
  fout.write("cid\ttid\tscore\tscore_note\tscore_type\terror\n")
  n=0; n_cid=0; n_dtp=0;
  tids_not_found = set()
  cids_not_found = set()
  for cid in cids:
    n_cid+=1
    if cid_skip and n_cid<=cid_skip: continue
    n_tid=0;
    for tid in tids:
      n_tid+=1
      if tid_skip and n_tid<=tid_skip: continue
      n+=1
      if cid in cids_not_found:
        logging.info("cid not found; skipping %s:%s"%(cid,tid))
        continue
      if tid in tids_not_found:
        logging.info("tid not found; skipping %s:%s"%(cid,tid))
        continue
      logging.info('%d.  [%d/%d][%d/%d]'%(n,n_cid,len(cids),n_tid,len(tids)))

      #######################################################################
      logging.info('%9s: %9s:'%(cid,tid))
      try:
        rval=rest.Utils.GetURL(base_url+'/%s:%s'%(cid,tid))
      except Exception as e:
        logging.error('HTTP Error (%s): %s'%(res,e))
      txt=('%s'%(str(rval)))
      logging.debug('rval="%s"'%txt)

      rob=re.compile(r'^(.*)(<graphml.*</graphml>).*$',re.I|re.DOTALL)
      score=""; score_note=""; score_type=""; errtxt=""; graphml="";
      m=rob.search(txt)
      if m:
        scorehtm=m.group(1)
        logging.debug('[cid=%s,tid=%s]: scorehtm: "%s"'%(cid,tid,scorehtm))
        graphml=m.group(2)
        rob2=re.compile(r'^.*P value: </i></b>([0-9+-e]+)\s*\(([^\)]+)\).*Type: </i></b><a.*>([^>]+)</a>.*$',re.I|re.DOTALL)
        rob3=re.compile(r'^.*P value: </i></b>([0-9+-e]+)\s*\(([^\)]+)\).*Type: </i></b>([^\n\r]+).*$',re.I|re.DOTALL)
        m2=rob2.search(scorehtm)
        m3=rob3.search(scorehtm)
        if m2:
          score=m2.group(1)
          score_note=m2.group(2)
          score_type=m2.group(3)
        elif m3:
          score=m3.group(1)
          score_note=m3.group(2)
          score_type=m3.group(3)
        logging.info('NOTE [cid=%s,tid=%s]: score=%s ; note="%s" ; type="%s"'%(cid,tid,score,score_note,score_type))
      else:
        errtxt=txt
        #if 'not found' in errtxt:
        logging.error('[cid=%s,tid=%s]: "%s"'%(cid,tid,errtxt))
      if odir and graphml.strip():
        ofile_graphml=(odir+"/%s_%s.graphml"%(cid,tid))
        fout_graphml=open(ofile_graphml,"w+")
        if not fout_graphml: logging.error('Failed to open output file: %s'%ofile_graphml)
        logging.info('%s'%ofile_graphml)
        fout_graphml.write(graphml+'\n')
        fout_graphml.flush()

      fout.write('%s,"%s",%s,"%s","%s","%s"\n'%(cid,tid,score,score_note,score_type,re.sub(r'[\n\r]+',r'\\n',errtxt)))
      fout.flush()
      #######################################################################

      if score:
        n_dtp+=1
        if score_type and not score_type in score_types: score_types[score_type]=1
        score_types[score_type]+=1
        if score_note and not score_note in score_notes: score_notes[score_note]=1
        score_notes[score_note]+=1
      if errtxt:
        if not errtxt in errs: errs[errtxt]=1
        errs[errtxt]+=1
        if errtxt == 'target was not found':
          tids_not_found.add(tid)
        elif errtxt == 'compound was not found':
          cids_not_found.add(cid)
      if tid_nmax and n_tid>=tid_skip+tid_nmax: break
    if cid_nmax and n_cid>=cid_skip+cid_nmax: break

  logging.info("drugs: %d targets: %d"%(n_cid-cid_skip,n_tid-tid_skip))
  logging.info("drug-target queries: %d  drug-target predictions: %d"%(n,n_dtp))
  for score_type in score_types.keys():
    logging.info('score_type "%s": %d'%(score_type,score_types[score_type]))
  for score_note in score_notes.keys():
    logging.info('score_note "%s": %d'%(score_note,score_notes[score_note]))
  for errtxt in errs.keys():
    logging.info('ERRORS: "%s": %d'%(errtxt,errs[errtxt]))

#############################################################################
def DDSimilarity(base_url,cid1,cid2,fout):
  '''Drug-drug similarity query'''
  htm = rest.Utils.GetURL(base_url+'/sim_%s:%s'%(cid1,cid2))
  logging.debug('htm = "%s"'%htm)

#############################################################################
#Kludge: workaround since py-dom-xpath does not allow nodes named "node"!
#############################################################################
def ParseDTPathGraphml(fin_xml):
  '''Parse drug-target graphml.'''
  xml_str=fin_xml.read()
  xml_str=re.sub(r'<node','<nnode',xml_str) #kludge
  xml_str=re.sub(r'/node>','/nnode>',xml_str) #kludge
  doc=None
  try:
    doc=ElementTree.fromstring(xml_str)
  except Exception as e:
    logging.error('XML parse error: %s'%e)
    return False
  return doc

#############################################################################
#We want to add a data node for gene name:
#
#<node id="http://chem2bio2rdf.org/uniprot/resource/gene/KCNJ11">
#       <data key="label">KCNJ11</data>
#       <data key="class">gene</data>
#       <data key="nodesize">1</data>
#       <data key="startnode">no</data>
#</node>
#
#       <data key="name">Potassium channel, inwardly rectifying, subfamily J, member 11</data>
#############################################################################
#"tid","protein_accession","chembl_id","tax_id","target_type","organism","db_source","gene_symbol","pref_name"
#"1","O43451","CHEMBL2074","9606","PROTEIN","Homo sapiens","SWISS-PROT","MGAM","Maltase-glucoamylase"
#"2","O60706","CHEMBL1971","9606","PROTEIN","Homo sapiens","SWISS-PROT","ABCC9","Sulfonylurea receptor 2"
#"3","O76074","CHEMBL1827","9606","PROTEIN","Homo sapiens","SWISS-PROT","PDE5A","Phosphodiesterase 5A"
#############################################################################
def AnnotateDTPathGraphmlTargets(doc,fin_csv):
  '''Annotate graphml with additional data from CSV file.'''

  n_in=0; n_out=0; errtxt=None;

  if not doc:
    return None

  ### Read CSV file containing target data:
  csvReader=csv.DictReader(
	fin_csv,
	fieldnames=None,
	restkey=None,
	restval=None,
	dialect='excel',
	delimiter=',',
	quotechar='"')
  try:
    csvrow=csvReader.next()    ## must do this to get fieldnames
    n_in+=1
  except:
    errtxt=('ERROR: bad ifile: %s'%fin_csv.name)
    return doc
  logging.debug('fieldnames=', csvReader.fieldnames)
  tgt_id_tag='protein_accession'
  tgt_gene_tag='gene_symbol'
  tgt_name_tag='pref_name'
  for tag in (tgt_id_tag,tgt_name_tag,tgt_gene_tag):
    if tag not in csvReader.fieldnames:
      logging.error('%s not in fieldnames'%tag)
      return doc

  tgt_id2name={};
  tgt_gene2id={};
  while True:
    try:
      csvrow=csvReader.next()
      n_in+=1
      tgt_id = csvrow[tgt_id_tag]
      tgt_name = csvrow[tgt_name_tag]
      tgt_gene = csvrow[tgt_gene_tag]
      tgt_gene2id[tgt_gene]=tgt_id
      tgt_id2name[tgt_id]=tgt_name
    except:
      break
  logging.debug('protein ids/names read: %d'%len(tgt_id2name))
  for tgt_id in tgt_id2name.keys():
    logging.debug('protein id: %s'%tgt_id)

  #gene_nodes = xpath.find('/graphml/graph/nnode/data[@key="class" and text()="gene"]/parent::node()', doc)
  gene_nodes = doc.findall('/graphml/graph/nnode/data[@key="class" and text()="gene"]/parent::node()')
  logging.debug('gene count: %d'%len(gene_nodes))

  for node in gene_nodes:
    tgt_id=None
    atts=node.attributes
    for cnode in node.childNodes:
      if cnode.attributes:
        for attName,attValue in cnode.attributes.items():
          if attName=='key' and attValue=='label':
            tgt_gene=cnode.text
            if tgt_gene in tgt_gene2id:
              tgt_id=tgt_gene2id[tgt_gene]
            logging.debug(': gene = "%s" ; tgt = "%s"'%(tgt_gene,tgt_id))

    if tgt_id and tgt_id in tgt_id2name:
      namenode=doc.createElement('data')
      namenode.setAttribute('key','name')
      namenode.appendChild(doc.createTextNode(tgt_id2name[tgt_id]))
      node.appendChild(namenode)
      xml_str=node.toprettyxml()
      xml_str=re.sub(r'\s*[\n\r]+',r'\n',xml_str)
      #xml_str=re.sub(r'<nnode (.*?)</nnode>',r'<node \1</node>',xml_str,re.DOTALL)
      xml_str=re.sub(r'<nnode','<node',xml_str)
      xml_str=re.sub(r'/nnode>','/node>',xml_str)
      logging.debug('%s'%xml_str)

  return doc

#############################################################################
# <node id="http://chem2bio2rdf.org/pubchem/resource/pubchem_compound/5472">
# 	<data key="label">5472</data>
# 	<data key="class">pubchem_compound</data>
# 	<data key="nodesize">1</data>
# 	<data key="startnode">no</data>
# </node>
#############################################################################
# cpd csv fields: cid, sids, synonyms
# sids and synonyms are semicolon-delimited
# synonyms only up to top ten "nicest"
#############################################################################
def AnnotateDTPathGraphmlCompounds(doc,fin_csv):
  '''Annotate graphml with additional data from CSV file.'''

  n_in=0; n_out=0; errtxt=None;

  if not doc: return None

  ### Read CSV file containing target data:
  csvReader=csv.DictReader(
	fin_csv,
	fieldnames=None,
	restkey=None,
	restval=None,
	dialect='excel',
	delimiter=',',
	quotechar='"')
  try:
    csvrow=csvReader.next()    ## must do this to get fieldnames
    n_in+=1
  except:
    errtxt=('ERROR: bad ifile: %s'%fin_csv.name)
    return doc
  logging.debug('fieldnames=', csvReader.fieldnames)

  cid_tag='cid'
  synonyms_tag='synonyms'
  for tag in (cid_tag,synonyms_tag):
    if tag not in csvReader.fieldnames:
      logging.error('%s not in fieldnames'%tag)
      return doc
  cid2synonyms={};
  n_synonyms_total=0;
  while True:
    try:
      csvrow=csvReader.next()
      n_in+=1
      cid = int(csvrow[cid_tag])
      synonyms = csvrow[synonyms_tag].split(';')
      cid2synonyms[cid]=synonyms
      n_synonyms_total+=len(synonyms)
    except:
      break
  logging.debug('cpd ids read: %d ; total synonyms: %d'%(len(cid2synonyms),n_synonyms_total))

  #Find cpd nodes in Graphml:
  #cpd_nodes=xpath.find('/graphml/graph/nnode/data[@key="class" and text()="pubchem_compound"]/parent::node()',doc)
  cpd_nodes=doc.findall('/graphml/graph/nnode/data[@key="class" and text()="pubchem_compound"]/parent::node()')
  n_cid=0;
  for node in cpd_nodes:
    cid=None
    atts=node.attributes
    for cnode in node.childNodes:
      if cnode.attributes:
        for attName,attValue in cnode.attributes.items():
          if attName=='key' and attValue=='label':
            try:
              cid = int(re.sub(r'\(.*\)','',cnode.text))
            except Exception as e:
              logging.error('%s'%e)
    if cid==None: continue
    n_cid+=1

    if cid in cid2synonyms:
      namenode=doc.createElement('data')
      namenode.setAttribute('key','synonyms')
      namenode.appendChild(doc.createTextNode('; '.join(cid2synonyms[cid])))
      node.appendChild(namenode)

      #Also revise cpd label:
      #cpd_label_node=xpath.find('data[@key="label"]',node)[0]
      cpd_label_node=node.find('data[@key="label"]')
      ReplaceNodeText(cpd_label_node, cid2synonyms[cid][0])

      xml_str=node.toprettyxml()
      xml_str=re.sub(r'\s*[\n\r]+',r'\n',xml_str)
      xml_str=re.sub(r'<nnode','<node',xml_str)
      xml_str=re.sub(r'/nnode>','/node>',xml_str)
      logging.debug('%s'%xml_str)

    else:
      logging.debug('cid %d not in csv'%cid)

  logging.info('graphml cid count: %d'%len(cid2synonyms))

  return doc

##############################################################################
def ReplaceNodeText(node, txt):
  if node.firstChild.nodeType != node.TEXT_NODE:
    raise Exception("node does not contain text")
  node.firstChild.replaceWholeText(txt)

##############################################################################
def DTPathGraphml2CIDs(doc,fout):
  #cpd_nodes=xpath.find('/graphml/graph/nnode/data[@key="class" and text()="pubchem_compound"]/parent::node()',doc)
  cpd_nodes=doc.findall('/graphml/graph/nnode/data[@key="class" and text()="pubchem_compound"]/parent::node()')
  n_cid=0;
  for node in cpd_nodes:
    cid=None
    atts=node.attributes
    for cnode in node.childNodes:
      if cnode.attributes:
        for attName,attValue in cnode.attributes.items():
          if attName=='key' and attValue=='label':
            try:
              cid = int(re.sub(r'\(.*\)', '', cnode.text))
            except Exception as e:
              logging.error('Error (Exception): %s'%e)
    if cid==None: continue
    fout.write('%d\n'%cid)
    n_cid+=1
  logging.info('cid count: %d'%n_cid)

##############################################################################
def DTPathGraphml2TIDs(doc,fout):
  #tgt_nodes=xpath.find('/graphml/graph/nnode/data[@key="class" and text()="gene"]/parent::node()',doc)
  tgt_nodes=doc.findall('/graphml/graph/nnode/data[@key="class" and text()="gene"]/parent::node()')
  n_tid=0;
  for node in tgt_nodes:
    tid=None
    atts=node.attributes
    for cnode in node.childNodes:
      if cnode.attributes:
        for attName,attValue in cnode.attributes.items():
          if attName=='key' and attValue=='label':
            try:
              tid = re.sub(r'\(.*\)', '', cnode.text)
            except Exception as e:
              logging.error('Error (Exception): %s'%e)
    if tid==None: continue
    fout.write('%s\n'%tid)
    n_tid+=1
  logging.info('tid count: %d'%n_tid)

##############################################################################
def MergeDTPathGraphmlsDir(idir_graphml):
  import glob
  i=0;
  doc_merged=None;
  for fpath in glob.glob(idir_graphml+'/*.graphml'):
    logging.debug('%s'%fpath)
    fin=open(fpath,"r")
    if not fin: logging.error('Failed to open input file: %s'%fpath)
    doc_this = ParseDTPathGraphml(fin)
    fin.close()
    if not doc_this:
      logging.error('Problem parsing: %s'%fpath)
      continue
    root_node = doc_this.documentElement
    child_nodes = []
    child_node = root_node.firstChild
    while child_node:
      child_nodes.append(child_node)
      child_node = child_node.nextSibling
    logging.debug('%s: root: %s  children: %s'%(fpath,root_node.tagName,','.join(map(lambda n:n.tagName,child_nodes))))
    if i==0:
      doc_merged=doc_this
    else:
      try:
        doc_merged = MergeDTPathGraphmlsPair(doc_merged,doc_this)
      except Exception as e:
        logging.error('(MergeDTPathGraphmlsDir): (Exception): %s'%e)
    i+=1
  logging.debug('merged graphmls: %d'%i)

  return doc_merged

##############################################################################
def MergeDTPathGraphmlsPair(docA,docB):
  '''Merge docB into docA and return docA.'''

  #graphA = xpath.find('/graphml/graph[1]', docA)
  graphA = docA.findall('/graphml/graph[1]')
  if not graphA:
    logging.debug('not graphA')
  else:
    graphA = graphA[0]
  #nodesA = xpath.find('/graphml/graph/nnode', docA)
  nodesA = docA.findall('/graphml/graph/nnode')
  #edgesA = xpath.find('/graphml/graph/edge', docA)
  edgesA = docA.findall('/graphml/graph/edge')
  if not edgesA:
    logging.debug('not edgesA')
  #edgeA1 = xpath.find('/graphml/graph/edge[1]', docA)
  edgeA1 = docA.find('/graphml/graph/edge[1]')
  edgeA1 = edgeA1[0]
  if not edgeA1:
    logging.debug('not edgeA1')

  #nodesB = xpath.find('/graphml/graph/nnode', docB)
  nodesB = docB.findall('/graphml/graph/nnode')
  #edgesB = xpath.find('/graphml/graph/edge', docB)
  edgesB = docB.findall('/graphml/graph/edge')

  for nodeB in nodesB:
    graphA.insertBefore(nodeB,edgeA1)
  for edgeB in edgesB:
    graphA.appendChild(edgeB)
  logging.debug('new nodes: %d edges: %d'%(len(nodesB),len(edgesB)))
  logging.debug('merged nodes: %d edges: %d'%(len(nodesA),len(edgesA)))

  return docA

##############################################################################
def ShowNodeClasses(doc):
  #nodes=xpath.find('/graphml/graph/nnode',doc)
  nodes=doc.findall('/graphml/graph/nnode')
  node_classes = {}
  for node in nodes:
    node_id=None
    node_class=None
    if node.attributes:
      for attName,attValue in node.attributes.items():
        if attName=='id':
          node_id = attValue
          node_class_uri = re.sub(r'/[^/]+$','',node_id)

    for cnode in node.childNodes:
      if cnode.nodeName=='data':
        if cnode.attributes:
          for attName,attValue in cnode.attributes.items():
            if attName=='key' and attValue=='class':
              node_classes[cnode.text] = node_class_uri
  logging.info('Node classes (%d):'%len(node_classes))
  for node_class in sorted(node_classes.keys()):
    logging.info('%18s:\t[%s]'%(node_class,node_classes[node_class]))
  return node_classes

##############################################################################
def ShowEdgeClasses(doc):
  #edges=xpath.find('/graphml/graph/edge',doc)
  edges=doc.findall('/graphml/graph/edge')
  edge_classes = {}
  for edge in edges:
    source_id=None
    source_class=None
    target_id=None
    target_class=None
    if edge.attributes:
      for attName,attValue in edge.attributes.items():
        if attName=='source':
          source_id = attValue
          source_class_uri = re.sub(r'/[^/]+$','',source_id)
        if attName=='target':
          target_id = attValue
          target_class_uri = re.sub(r'/[^/]+$','',target_id)

    for cnode in edge.childNodes:
      if cnode.nodeName=='data':
        if cnode.attributes:
          for attName,attValue in cnode.attributes.items():
            if attName=='key' and attValue=='uri':
              edge_class = cnode.text
              if not edge_class in edge_classes:
                edge_classes[edge_class] = set()
              edge_classes[edge_class].add(tuple(sorted([source_class_uri,target_class_uri])))

  logging.info('Edge classes (%d):'%len(edge_classes))
  for edge_class in sorted(edge_classes.keys()):
    edge_class_name = re.sub(r'^.*/','',edge_class)
    logging.info('%18s:\t[%s]'%(edge_class_name,edge_class))
    for source,target in edge_classes[edge_class]:
      source_name = re.sub(r'^.*/','',source)
      target_name = re.sub(r'^.*/','',target)
      logging.info('\t\t%s : %s'%(source_name,target_name))
  return edge_classes

##############################################################################
def DTPathGraphmlSummary(doc):
  #nodes=xpath.find('/graphml/graph/nnode',doc)
  nodes=doc.findall('/graphml/graph/nnode')
  logging.info('Nodes: %d'%len(nodes))
  #edges=xpath.find('/graphml/graph/edge',doc)
  edges=doc.findall('/graphml/graph/edge')
  logging.info('Edges: %d'%len(edges))

##############################################################################
#Kludge: workaround since py-dom-xpath does not allow nodes named "node"!
def GraphmlDocWrite(doc,fout):
  xml_str = doc.toprettyxml()
  xml_str=re.sub(r'\s*[\n\r]+',r'\n',xml_str)
  #xml_str=re.sub(r'<nnode (.*?)</nnode>',r'<node \1</node>',xml_str,re.DOTALL)
  xml_str=re.sub(r'<nnode','<node',xml_str)
  xml_str=re.sub(r'/nnode>','/node>',xml_str)
  fout.write(xml_str)

##############################################################################
if __name__=='__main__':
  epilog='''\
dtp = Drug Target Prediction;
dtp_annotate_tgts: annotate GraphML (tgts required);
dtp_annotate_cpds: annotate GraphML (cpds required);
dtp_annotate: annotate GraphML (tgts and cpds required);
graphml2cids: find, list PubChem CIDs;
graphml2tids: find, list (Uniprot gene) TIDs;
graphmls_merge: merge all GraphMLs in --i_graphml_dir ;
show_node_classes: show all node classes ;
show_edge_classes: show all edge classes;
summary: summary of GraphML contents
'''
  parser = argparse.ArgumentParser(description="SLAP results GraphML post-processing utility", epilog=epilog)
  ops = [ "summary", "dtp_annotate", "dtp_annotate_tgts", "dtp_annotate_cpds", "graphml2cids", "graphml2tids", "graphmls_merge", "show_node_classes", "show_edge_classes" ]
  parser.add_argument("op", choices=ops, help="operation")
  parser.add_argument("--i_graphml", dest="ifile_graphml", help="input file (GRAPHML)")
  parser.add_argument("--i_graphml_dir", dest="idir_graphml", help="input dir (GRAPHML)")
  parser.add_argument("--i_tgts", dest="ifile_tgts", help="input file, target IDs (CSV)")
  parser.add_argument("--i_cpds", dest="ifile_cpds", help="input file, compound IDs (CSV)")
  parser.add_argument("--o", dest="ofile", help="output (TSV or GraphML)")
  parser.add_argument("-v", "--verbose", dest="verbose", action="count", default=0)
  args = parser.parse_args()

  logging.basicConfig(format='%(levelname)s:%(message)s', level=(logging.DEBUG if args.verbose>1 else logging.INFO))

  fout = open(args.ofile, "w+") if args.ofile else sys.stdout

  fin_graphml = open(args.ifile_graphml, "r") if args.ifile_graphml else None
  fin_tgts = open(args.ifile_tgts, "r") if args.ifile_tgts else None
  fin_cpds = open(args.ifile_cpds, "r") if args.ifile_cpds else None

  t0=time.time()

  if args.op=="dtp_annotate":
    doc = ParseDTPathGraphml(fin_graphml)
    doc = AnnotateDTPathGraphmlTargets(doc, fin_tgts)
    doc = AnnotateDTPathGraphmlCompounds(doc, fin_cpds)
    GraphmlDocWrite(doc, fout)

  elif args.op=="dtp_annotate_tgts":
    doc = ParseDTPathGraphml(fin_graphml)
    doc = AnnotateDTPathGraphmlTargets(doc, fin_tgts)
    GraphmlDocWrite(doc, fout)

  elif args.op=="dtp_annotate_cpds":
    doc = ParseDTPathGraphml(fin_graphml)
    doc = AnnotateDTPathGraphmlCompounds(doc, fin_cpds)
    GraphmlDocWrite(doc, fout)

  elif args.op=="graphml2cids":
    doc = ParseDTPathGraphml(fin_graphml)
    DTPathGraphml2CIDs(doc, fout)

  elif args.op=="graphml2tids":
    doc = ParseDTPathGraphml(fin_graphml)
    DTPathGraphml2TIDs(doc, fout)

  elif args.op=="graphmls_merge":
    doc = MergeDTPathGraphmlsDir(idir_graphml)
    GraphmlDocWrite(doc, fout)

  elif args.op=="show_node_classes":
    doc = ParseDTPathGraphml(fin_graphml)
    node_classes=ShowNodeClasses(doc)

  elif args.op=="show_edge_classes":
    doc = ParseDTPathGraphml(fin_graphml)
    edge_classes=ShowEdgeClasses(doc)

  elif args.op=="summary":
    doc = ParseDTPathGraphml(fin_graphml)
    DTPathGraphmlSummary(doc)

  else:
    parser.error("Invalid operation: {0}".format(args.op))

  logging.info('Elapsed time: %s'%(time.strftime('%Hh:%Mm:%Ss', time.gmtime(time.time()-t0))))



#!/usr/bin/env python3
"""
Pandas CSV/TSV utilities.
"""
###
import sys,os,argparse,re,pickle,logging
import pandas as pd

from .. import pandas as util_pandas

#############################################################################
if __name__=='__main__':
  verstr = (f'Python: {sys.version.split()[0]}; pandas: {pd.__version__}')
  parser = argparse.ArgumentParser(prog="BioClients.util.pandas.Utils", description='Pandas utilities for simple datafile transformations.', epilog=verstr)
  ops = ['csv2tsv', 'shape', 'summary', 'showcols', 'list_columns', 'to_html',
	'selectcols', 'selectcols_deduplicate', 'uvalcounts',
	'colvalcounts', 'sortbycols', 'deduplicate', 'colstats', 'searchrows',
	'pickle', 'sample', 'set_header', 'remove_header']
  compressions=['gzip', 'zip', 'bz2']
  parser.add_argument("op", choices=ops, help='OPERATION')
  parser.add_argument("--i", dest="ifile", required=True, help="input (CSV|TSV)")
  parser.add_argument("--o", dest="ofile", help="output (CSV|TSV)")
  parser.add_argument("--coltags", help="cols specified by tag (comma-separated)")
  parser.add_argument("--cols", help="cols specified by idx (1+) (comma-separated)")
  parser.add_argument("--search_qrys", help="qrys (comma-separated, NA|NaN handled specially)")
  parser.add_argument("--search_rels", default="=", help="relationships (=|>|<) (comma-separated)")
  parser.add_argument("--search_typs", default="str", help="types (str|int|float) (comma-separated)")
  parser.add_argument("--compression", choices=compressions)
  parser.add_argument("--csv", action="store_true", help="delimiter is comma")
  parser.add_argument("--tsv", action="store_true", help="delimiter is tab")
  parser.add_argument("--on_bad_lines", choices=["error", "warn", "skip"], default="warn")
  parser.add_argument("--nrows", type=int)
  parser.add_argument("--skiprows", type=int)
  parser.add_argument("--sample_frac", type=float, default=.01, help="sampling probability (0-1)")
  parser.add_argument("--sample_n", type=int, help="sampling N")
  parser.add_argument("--html_title", help="table title")
  parser.add_argument("--html_prettify", action="store_true", help="requires Pandas v1.4.0+")
  parser.add_argument("-v", "--verbose", action="count", default=0)
  args = parser.parse_args()

  logging.basicConfig(format='%(levelname)s:%(message)s', level=(logging.DEBUG if args.verbose>1 else logging.INFO))

  if args.op in ('selectcols', 'uvalcounts', 'colvalcounts', 'sortbycols'):
    if not (args.cols or args.coltags): 
      parser.error(f'{args.op} requires --cols or --coltags.')

  logging.info(verstr)

  fout = open(args.ofile, "w") if args.ofile else sys.stdout

  if args.compression: compression=args.compression
  elif re.search('\.gz$', args.ifile, re.I): compression='gzip'
  elif re.search('\.bz2$', args.ifile, re.I): compression='bz2'
  elif re.search('\.zip$', args.ifile, re.I): compression='zip'
  else: compression=None

  if args.csv or args.op=='csv2tsv': delim=','
  elif args.tsv: delim='\t'
  elif re.search('\.csv', args.ifile, re.I): delim=','
  elif re.search('\.tsv', args.ifile, re.I) or re.search('\.tab', args.ifile, re.I): delim='\t'
  else: delim='\t'

  cols=None; coltags=None;
  if args.cols:
    cols = [(int(col.strip())-1) for col in re.split(r',', args.cols.strip())]
  elif args.coltags:
    coltags = [coltag.strip() for coltag in re.split(r',', args.coltags.strip())]

  search_qrys = [qry.strip() for qry in re.split(r',', args.search_qrys.strip())] if (args.search_qrys is not None) else None
  search_rels = [rel.strip() for rel in re.split(r',', args.search_rels.strip())] if (args.search_rels is not None) else None
  search_typs = [typ.strip() for typ in re.split(r',', args.search_typs.strip())] if (args.search_typs is not None) else None

  df = pd.read_csv(args.ifile, sep=delim, compression=compression,
on_bad_lines=args.on_bad_lines, nrows=(1 if args.op in ('showcols', 'list_columns') else args.nrows), skiprows=args.skiprows)

  if args.op == 'showcols':
    for j,tag in enumerate(df.columns):
      fout.write(f'{j+1}. "{tag}"\n')

  elif args.op == 'list_columns':
    fout.write("\n".join(df.columns)+"\n")

  elif args.op == 'shape':
    fout.write(f"rows: {df.shape[0]}; cols: {df.shape[1]}\n")

  elif args.op == 'summary':
    fout.write(f"rows: {df.shape[0]}; cols: {df.shape[1]}\n")
    fout.write("coltags: {}\n".format(', '.join([f'"{tag}"' for tag in df.columns])))

  elif args.op=='csv2tsv':
    df.to_csv(fout, '\t', index=False)

  elif args.op=='to_html':
    util_pandas.ToHtml(df, args.html_title, args.html_prettify, fout)

  elif args.op == 'selectcols':
    logging.info(f"Input: rows: {df.shape[0]}; cols: {df.shape[1]}")
    df = df[coltags] if coltags else df.iloc[:, cols]
    logging.info(f"Output: rows: {df.shape[0]}; cols: {df.shape[1]}")
    df.to_csv(fout, '\t', index=False)

  elif args.op == 'selectcols_deduplicate':
    logging.info(f"Input: rows: {df.shape[0]}; cols: {df.shape[1]}")
    df = df[coltags] if coltags else df.iloc[:, cols]
    df.drop_duplicates(inplace=True)
    logging.info(f"Output: rows: {df.shape[0]}; cols: {df.shape[1]}")
    df.to_csv(fout, '\t', index=False)

  elif args.op == 'uvalcounts':
    for j,tag in enumerate(df.columns):
      if cols and j not in cols: continue
      if coltags and tag not in coltags: continue
      logging.info(f'{j+1}. {tag}: {df[tag].nunique()}')

  elif args.op == 'colvalcounts':
    for j,tag in enumerate(df.columns):
      if cols and j not in cols: continue
      if coltags and tag not in coltags: continue
      logging.info(f'{j+1}. {tag}:')
      for key,val in df[tag].value_counts().iteritems():
        logging.info(f'\t{tag}: {val:6d}: {key}')

  elif args.op == 'colstats':
    for j,tag in enumerate(df.columns):
      if cols and j not in cols: continue
      if coltags and tag not in coltags: continue
      fout.write(f'{j+1}. {tag}:\n')
      fout.write(f'\tN: {df[tag].size}\n')
      fout.write(f'\tN_isna: {df[tag].isna().sum()}\n')
      fout.write(f'\tmin: {df[tag].min():.2f}\n')
      fout.write(f'\tmax: {df[tag].max():.2f}\n')
      fout.write(f'\tmean: {df[tag].mean():.2f}\n')
      fout.write(f'\tmedian: {df[tag].median():.2f}\n')
      fout.write(f'\tstd: {df[tag].std():.2f}\n')

  elif args.op == 'deduplicate':
    df.drop_duplicates(inplace=True)
    df.to_csv(fout, '\t', index=False)

  elif args.op == 'searchrows':
    if args.search_qrys is None: 
      parser.error(f'{args.op} requires --search_qrys.')
    logging.debug(f"search_qrys={search_qrys}")
    logging.debug(f"search_rels={search_rels}")
    logging.debug(f"search_typs={search_typs}")
    util_pandas.SearchRows(df, cols, coltags, search_qrys, search_rels, search_typs, fout)

  elif args.op == 'sample':
    util_pandas.SampleRows(df, args.sample_frac, args.sample_n, fout)

  elif args.op == 'set_header':
    util_pandas.SetHeader(df, coltags, delim, fout)

  elif args.op == 'remove_header':
    util_pandas.RemoveHeader(df, delim, fout)

  elif args.op == 'pickle':
    if not args.ofile:
      parser.error(f'{args.op} requires --o.')
    fout.close()
    with open(args.ofile, 'wb') as fout:
      pickle.dump(df, fout, pickle.HIGHEST_PROTOCOL)

  else:
    parser.error(f'Unknown operation: {args.op}')
